#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
__all__ = ["RHEL3Handler"]

from pykickstart import commands
from pykickstart.base import BaseHandler
from pykickstart.version import RHEL3

class RHEL3Handler(BaseHandler):
    version = RHEL3

    commandMap = {
        "auth": commands.authconfig.FC3_Authconfig,
        "authconfig": commands.authconfig.FC3_Authconfig,
        "autopart": commands.autopart.FC3_AutoPart,
        "autostep": commands.autostep.FC3_AutoStep,
        "bootloader": commands.bootloader.FC3_Bootloader,
        "cdrom": commands.cdrom.FC3_Cdrom,
        "clearpart": commands.clearpart.FC3_ClearPart,
        "cmdline": commands.displaymode.FC3_DisplayMode,
        "device": commands.device.FC3_Device,
        "deviceprobe": commands.deviceprobe.FC3_DeviceProbe,
        "driverdisk": commands.driverdisk.FC3_DriverDisk,
        "firewall": commands.firewall.FC3_Firewall,
        "firstboot": commands.firstboot.FC3_Firstboot,
        "graphical": commands.displaymode.FC3_DisplayMode,
        "halt": commands.reboot.FC3_Reboot,
        "harddrive": commands.harddrive.FC3_HardDrive,
        "ignoredisk": commands.ignoredisk.FC3_IgnoreDisk,
        "install": commands.upgrade.FC3_Upgrade,
        "interactive": commands.interactive.FC3_Interactive,
        "keyboard": commands.keyboard.FC3_Keyboard,
        "lang": commands.lang.FC3_Lang,
        "langsupport": commands.langsupport.FC3_LangSupport,
        "lilo": commands.bootloader.FC3_Lilo,
        "lilocheck": commands.lilocheck.FC3_LiloCheck,
        "logvol": commands.logvol.FC3_LogVol,
        "method": commands.method.FC3_Method,
        "monitor": commands.monitor.FC3_Monitor,
        "mouse": commands.mouse.RHEL3_Mouse,
        "network": commands.network.FC3_Network,
        "nfs": commands.nfs.FC3_NFS,
        "part": commands.partition.FC3_Partition,
        "partition": commands.partition.FC3_Partition,
        "poweroff": commands.reboot.FC3_Reboot,
        "raid": commands.raid.FC3_Raid,
        "reboot": commands.reboot.FC3_Reboot,
        "rootpw": commands.rootpw.FC3_RootPw,
        "shutdown": commands.reboot.FC3_Reboot,
        "skipx": commands.skipx.FC3_SkipX,
        "text": commands.displaymode.FC3_DisplayMode,
        "timezone": commands.timezone.FC3_Timezone,
        "upgrade": commands.upgrade.FC3_Upgrade,
        "url": commands.url.FC3_Url,
        "vnc": commands.vnc.FC3_Vnc,
        "volgroup": commands.volgroup.FC3_VolGroup,
        "xconfig": commands.xconfig.FC3_XConfig,
        "zerombr": commands.zerombr.FC3_ZeroMbr,
    }

    dataMap = {
        "DriverDiskData": commands.driverdisk.FC3_DriverDiskData,
        "LogVolData": commands.logvol.FC3_LogVolData,
        "NetworkData": commands.network.RHEL4_NetworkData,
        "PartData": commands.partition.FC3_PartData,
        "RaidData": commands.raid.FC3_RaidData,
        "VolGroupData": commands.volgroup.FC3_VolGroupData,
        "ZFCPData": commands.zfcp.FC3_ZFCPData,
    }



#
# Copyright 2019 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser
from pykickstart.version import RHEL8

from pykickstart.i18n import _

class RHEL8_RHSM(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.organization = kwargs.get("organization", None)
        self.activation_keys = kwargs.get("activation_keys", None)
        self.connect_to_insights = kwargs.get("connect_to_insights", None)
        self.proxy = kwargs.get("proxy", None)
        self.server_hostname = kwargs.get("server_hostname", None)
        self.rhsm_baseurl = kwargs.get("rhsm_baseurl", None)


    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if not retval and not any([self.organization, self.activation_keys,
                                   self.proxy, self.server_hostname,
                                   self.rhsm_baseurl]):
            return ""

        retval += '# Red Hat Subscription Manager\nrhsm'
        # TODO: check if including the organization name & activation key in
        #       the output kickstart is expected and safe
        if self.organization:
            retval+=' --organization="%s"' % self.organization
        for key in self.activation_keys:
            retval+=' --activation-key="%s"' % key
        if self.connect_to_insights:
            retval+=' --connect-to-insights'
        if self.proxy:
            retval += ' --proxy="%s"' % self.proxy
        if self.server_hostname:
            retval+=' --server-hostname="%s"' % self.server_hostname
        if self.rhsm_baseurl:
            retval+=' --rhsm-baseurl="%s"' % self.rhsm_baseurl
        retval+='\n'
        return retval

    def _getParser(self):
        op = KSOptionParser(prog="rhsm", description="""
                            The rhsm command is used to configure a Red Hat subscription.
                            A Red Hat subscription is required for installing from
                            the Red Hat CDN installation source as well connecting the target
                            system to Red Hat Insights.

                            Subscription attached during the installation will also be available
                            on the target system after the installation.

                            System Purpose data specified via the syspurpose command may influence
                            what subscription will be attached to the system. System Purpose data
                            set via the syspurpose command will override system purpose data attached
                            to activation keys.

                            An organization id needs to be specified, as well as at least one activation key.
                            """, version=RHEL8)
        op.add_argument("--organization", metavar="<organization_name>", version=RHEL8, required=True,
                        help="Organization id.")
        op.add_argument("--activation-key", metavar="<activation_key>", action="append", dest="activation_keys",
                        version=RHEL8, required=True,
                        help="Activation key. Option can be used multiple times, once per activation key.")
        op.add_argument("--connect-to-insights", version=RHEL8, action="store_true", default=False,
                        help="Connect to Red Hat Insights.")
        op.add_argument("--proxy", version=RHEL8, help="""
                        Specify an HTTP proxy to use for subscription purposes.
                        The syntax is::

                        ``--proxy=[protocol://][username[:password]@]host[:port]``
                        """)
        op.add_argument("--server-hostname", metavar="<subscription_service_server_url>", version=RHEL8, required=False,
                        help="Red Hat subscription service server URL.")
        op.add_argument("--rhsm-baseurl", metavar="<content_base_url>", version=RHEL8, required=False,
                        help="Content base URL.")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)

        for key in ns.activation_keys:
            if key == "":
                raise KickstartParseError(_("Empty string is not a valid activation key."), lineno=self.lineno)

        self.set_to_self(ns)
        return self



#!/usr/bin/env python3
#
# Simple script to take a kickstart config, read it in, parse any %includes,
# etc to write out a flattened config that is stand-alone
#
# Copyright 2007-2014, Red Hat, Inc.
# Jeremy Katz <katzj@redhat.com>
# Chris Lumens <clumens@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from __future__ import print_function

import sys
import argparse
import pykickstart
import pykickstart.parser
from pykickstart.i18n import _
from pykickstart.version import DEVEL, makeVersion
from pykickstart.errors import KickstartVersionError

def parse_args(argv):
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--config", dest="kscfg", required=True,
                        help=_("Path to kickstart config file"))
    parser.add_argument("-v", "--version", dest="version", default=DEVEL,
                        help=_("Kickstart version to use for interpreting config"))
    parser.add_argument("-o", "--output", dest="output",
                        help=_("Write flattened config to OUTPUT"))

    return parser.parse_args(argv)

def main(argv=None):
    opts = parse_args(argv)
    if not opts.kscfg:
        return (1, _("Need to specify a config to flatten"))

    try:
        ksversion = makeVersion(opts.version)
    except KickstartVersionError:
        print(_("The version %s is not supported by pykickstart") % opts.version)
        sys.exit(1)

    ksparser = pykickstart.parser.KickstartParser(ksversion)
    try:
        ksparser.readKickstart(opts.kscfg)
    except IOError as msg:
        return (1, _("Failed to read kickstart file '%(filename)s' : %(error_msg)s") % {"filename": opts.kscfg, "error_msg": msg})
    except pykickstart.errors.KickstartError as e:
        return (1, _("Failed to parse kickstart file '%(filename)s' : %(error_msg)s") % {"filename": opts.kscfg, "error_msg": e})

    if opts.output:
        try:
            f = open(opts.output, 'w')
        except IOError as msg:
            return (1, _("Failed to open output file '%(filename)s' : %(error_msg)s") % {"filename": opts.output, "error_msg": msg})
    else:
        f = sys.stdout

    f.write("%s" % ksparser.handler)

    if opts.output:
        f.close()

    return (0, '')

if __name__ == "__main__":
    retval, msg = main()
    if msg:
        print(msg, file=sys.stderr)
    sys.exit(retval)



#!/usr/bin/env python3

import sys

from pocketlint import FalsePositive, PocketLintConfig, PocketLinter

class PykickstartLintConfig(PocketLintConfig):
    def __init__(self):
        PocketLintConfig.__init__(self)

        self.falsePositives = [
            FalsePositive(r"^W1113.*: Keyword argument before variable positional arguments list in the definition of __init__ function$"),
            FalsePositive("W0707.*raise-missing-from"),
        ]

    @property
    def ignoreNames(self):
        return {"translation-canary", ".tox"}

if __name__ == "__main__":
    conf = PykickstartLintConfig()
    linter = PocketLinter(conf)
    rc = linter.run()
    if rc in [0, 4]:
        sys.exit(0)
    else:
        sys.exit(rc)



#
# Radek Vykydal <rvykydal@redhat.com>
#
# Copyright 2013 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest

from pykickstart.errors import KickstartParseWarning
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.commands.network import FC3_NetworkData, FC4_NetworkData, \
    FC6_NetworkData, F16_NetworkData, F25_NetworkData, \
    RHEL4_NetworkData, RHEL6_NetworkData, RHEL7_NetworkData
from pykickstart.constants import BIND_TO_MAC
from pykickstart.version import FC3

class Network_TestCase(unittest.TestCase):
    def runTest(self):
        fc3_data1 = FC3_NetworkData()
        fc3_data2 = FC3_NetworkData()
        # test default values
        self.assertFalse(fc3_data1.nodns)
        self.assertTrue(fc3_data1.onboot)

        # test equality/non-equality
        self.assertEqual(fc3_data1, fc3_data2)
        self.assertFalse(fc3_data1 != fc3_data2) # test the __ne__ method
        self.assertNotEqual(fc3_data1, None)

        for attr in ['device']:
            setattr(fc3_data1, attr, '')
            setattr(fc3_data2, attr, 'test')
            self.assertNotEqual(fc3_data1, fc3_data2)
            self.assertNotEqual(fc3_data2, fc3_data1)
            setattr(fc3_data1, attr, '')
            setattr(fc3_data2, attr, '')

        fc4_data = FC4_NetworkData()
        self.assertFalse(fc4_data.notksdevice)

        fc6_data = FC6_NetworkData()
        self.assertFalse(fc6_data.noipv4)
        self.assertFalse(fc6_data.noipv6)

        f16_data = F16_NetworkData()
        self.assertFalse(f16_data.activate)
        self.assertFalse(f16_data.nodefroute)

        f25_data = F25_NetworkData(activate=False)
        self.assertTrue(f25_data._getArgsAsStr().find('--no-activate') > -1)

        rhel4_data = RHEL4_NetworkData()
        self.assertFalse(rhel4_data.notksdevice)

        rhel6_data = RHEL6_NetworkData()
        self.assertFalse(rhel6_data.activate)
        self.assertFalse(rhel6_data.nodefroute)

        rhel7_data = RHEL7_NetworkData(activate=False)
        self.assertTrue(rhel7_data._getArgsAsStr().find('--no-activate') > -1)


class FC3_TestCase(CommandTest):
    def __init__(self, *kargs, **kwargs):
        CommandTest.__init__(self, *kargs, **kwargs)
        self.bootProtos = ["dhcp", "bootp", "static"]

    def runTest(self):
        # equality
        self.assertEqual(self.assert_parse("network --device=eth0"), self.assert_parse("network --device=eth0"))
        self.assertNotEqual(self.assert_parse("network --device=eth0"), None)
        self.assertNotEqual(self.assert_parse("network --device=eth0"), self.assert_parse("network --device=eth1"))

        # pass
        self.assert_parse("network --device=eth0 --dhcpclass CLASS",
                          "network  --bootproto=dhcp --dhcpclass=CLASS --device=eth0\n")
        self.assert_parse("network --device=eth0 --essid ESSID --wepkey WEPKEY",
                          "network  --bootproto=dhcp --device=eth0 --essid=\"ESSID\" --wepkey=WEPKEY\n")
        self.assert_parse("network --device=eth0 --ethtool \"gro on\" --mtu=1200",
                          "network  --bootproto=dhcp --device=eth0 --ethtool=\"gro on\" --mtu=1200\n")
        self.assert_parse("network --device=eth0 --gateway gateway.wherever.com --hostname server.wherever.com",
                          "network  --bootproto=dhcp --device=eth0 --gateway=gateway.wherever.com --hostname=server.wherever.com\n")
        self.assert_parse("network --device=eth0 --ip 1.2.3.4 --netmask 255.255.255.0",
                          "network  --bootproto=dhcp --device=eth0 --ip=1.2.3.4 --netmask=255.255.255.0\n")
        self.assert_parse("network --device=eth0 --nameserver ns.wherever.com",
                          "network  --bootproto=dhcp --device=eth0 --nameserver=ns.wherever.com\n")
        self.assert_parse("network --device=eth0 --nodns",
                          "network  --bootproto=dhcp --device=eth0 --nodns\n")
        self.assert_parse("network --device=eth0 --onboot=off",
                          "network  --bootproto=dhcp --device=eth0 --onboot=off\n")

        for bp in self.bootProtos:
            self.assert_parse("network --device=eth0 --bootproto=%s" % bp,
                              "network  --bootproto=%s --device=eth0\n" % bp)

        # fail - invalid bootproto
        self.assert_parse_error("network --device=eth0 --bootproto=bogus")

        # fail - invalid option
        self.assert_parse_error("network --bogus-option")

        # extra test coverage
        nic = self.handler().NetworkData(device='eth0')
        cmd = self.handler().commands['network']
        self.assertEqual(cmd.__str__(), "")
        cmd.network.append(nic)
        self.assertEqual(cmd.__str__(), "# Network information\nnetwork  --bootproto=dhcp --device=eth0\n")


class FC3_Duplicate_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = FC3

    def runTest(self):
        self.assert_parse("""
network --device=eth0
network --device=eth1""")

        self.assert_parse_error("""
network --device=eth0
network --device=eth0""", KickstartParseWarning)

class RHEL4_TestCase(FC3_TestCase):
    def runTest(self):
        FC3_TestCase.runTest(self)

        self.assert_parse("network --notksdevice",
                          "network  --bootproto=dhcp --notksdevice\n")

class FC4_TestCase(FC3_TestCase):
    def runTest(self):
        FC3_TestCase.runTest(self)

        self.assert_parse("network --notksdevice",
                          "network  --bootproto=dhcp --notksdevice\n")

class FC6_TestCase(FC4_TestCase):
    def runTest(self):
        FC4_TestCase.runTest(self)

        # bootproto is removed if --noipv4 is given in F24.
        if isinstance(self, F24_TestCase):
            self.assert_parse("network --device=eth0 --noipv4",
                              "network  --device=eth0 --noipv4\n")
        else:
            self.assert_parse("network --device=eth0 --noipv4",
                              "network  --bootproto=dhcp --device=eth0 --noipv4\n")

        self.assert_parse("network --device=eth0 --noipv6",
                          "network  --bootproto=dhcp --device=eth0 --noipv6\n")

class F8_TestCase(FC6_TestCase):
    def runTest(self):
        FC6_TestCase.runTest(self)

        self.assert_parse("network --device=eth0 --ipv6=1:2:3:4",
                          "network  --bootproto=dhcp --device=eth0 --ipv6=1:2:3:4\n")

class RHEL6_TestCase(F8_TestCase):
    def runTest(self):
        F8_TestCase.runTest(self)

        self.assert_parse("network --device=eth0 --activate",
                          "network  --bootproto=dhcp --device=eth0 --activate\n")
        self.assert_parse("network --device=eth0 --nodefroute",
                          "network  --bootproto=dhcp --device=eth0 --nodefroute\n")
        self.assert_parse("network --device=eth0 --bondslaves=A,B --bondopts=opt1,opt2",
                          "network  --bootproto=dhcp --device=eth0 --bondslaves=A,B --bondopts=opt1,opt2\n")
        self.assert_parse("network --device=eth0 --vlanid=ID",
                          "network  --bootproto=dhcp --device=eth0 --vlanid=ID\n")

class F9_TestCase(F8_TestCase):
    def __init__(self, *kargs, **kwargs):
        F8_TestCase.__init__(self, *kargs, **kwargs)
        self.bootProtos.append("query")

class F16_TestCase(F9_TestCase):
    def __init__(self, *kargs, **kwargs):
        F9_TestCase.__init__(self, *kargs, **kwargs)
        self.bootProtos.append("ibft")

    def runTest(self):
        F9_TestCase.runTest(self)

        self.assert_parse("network --device=eth0 --activate",
                          "network  --bootproto=dhcp --device=eth0 --activate\n")
        self.assert_parse("network --device=eth0 --nodefroute",
                          "network  --bootproto=dhcp --device=eth0 --nodefroute\n")
        self.assert_parse("network --device=eth0 --wpakey WPAKEY",
                          "network  --bootproto=dhcp --device=eth0 --wpakey=WPAKEY\n")

class F18_TestCase(F16_TestCase):
    def runTest(self):
        F16_TestCase.runTest(self)

        cmd = self.handler().commands['network']
        self.assertEqual(cmd.hostname, None)

        cmd.network.append(self.handler().NetworkData(device='eth0'))
        cmd.network.append(
            self.handler().NetworkData(device='eth1', hostname='example.com')
        )
        self.assertEqual(cmd.hostname, 'example.com')


class F19_TestCase(F18_TestCase):
    def runTest(self):
        F18_TestCase.runTest(self)

        self.assert_parse("network --device=eth0 --bondslaves=A,B --bondopts=opt1,opt2",
                          "network  --bootproto=dhcp --device=eth0 --bondslaves=A,B --bondopts=opt1,opt2\n")
        self.assert_parse("network --device=eth0 --vlanid=ID",
                          "network  --bootproto=dhcp --device=eth0 --vlanid=ID\n")
        self.assert_parse("network --device=eth0 --ipv6gateway=gateway6.wherever.com",
                          "network  --bootproto=dhcp --device=eth0 --ipv6gateway=gateway6.wherever.com\n")

class F20_TestCase(F19_TestCase):
    def runTest(self):
        F19_TestCase.runTest(self)

        # team device

        cmd = "network --device team0 --bootproto static --ip=10.34.102.222 --netmask=255.255.255.0 --gateway=10.34.102.254 --nameserver=10.34.39.2 --teamslaves=\"p3p1'{\\\"prio\\\": -10, \\\"sticky\\\": true}',p3p2'{\\\"prio\\\": 100}'\" --teamconfig=\"{\\\"runner\\\": {\\\"name\\\": \\\"activebackup\\\"}}\" --activate"
        self.assert_parse(cmd)

        cmd = "network --device team0 --bootproto dhcp --teamslaves=p3p1,p3p2 --teamconfig=\"{\\\"runner\\\": {\\\"name\\\": \\\"roundrobin\\\"}}\" --activate"
        outputCmd = "network  --bootproto=dhcp --device=team0 --activate --teamslaves=\"p3p1,p3p2\" --teamconfig=\"{\\\"runner\\\": {\\\"name\\\": \\\"roundrobin\\\"}}\"\n"
        self.assert_parse(cmd, outputCmd)

        cmd = "network --device team0 --bootproto dhcp --teamslaves=''"
        outputCmd = "network  --bootproto=dhcp --device=team0\n"
        self.assert_parse(cmd, outputCmd)

        # --teamslaves
        # --teamslaves="<DEV1>['<CONFIG1>'],<DEV2>['<CONFIG2>'],..."
        # CONFIGX is json with " escaped to \"

        teamslaves_strings = [
            r"eth1,eth2",
            r"eth1,eth2'{\"prio\": 100}'",
            r"eth1'{\"prio\": -10, \"sticky\": true}',eth2",
            r"eth1'{\"prio\": -10, \"sticky\": true}',eth2'{\"prio\": 100}'",
            r"eth1,eth2,eth3",
            r"eth1,eth2'{\"prio\": 100}',eth3",
        ]

        teamslaves_values = [
            [('eth1', ""), ('eth2', "")],
            [('eth1', ""), ('eth2', '{"prio": 100}')],
            [('eth1', '{"prio": -10, "sticky": true}'), ('eth2', "")],
            [('eth1', '{"prio": -10, "sticky": true}'), ('eth2', '{"prio": 100}')],
            [('eth1', ""), ('eth2', ""), ('eth3', "")],
            [('eth1', ""), ('eth2', '{"prio": 100}'), ('eth3', "")],
        ]

        for string, value in zip(teamslaves_strings, teamslaves_values):
            network_data = self.assert_parse("network --device team0 --bootproto=dhcp --teamslaves=\"%s\"" % string)
            self.assertEqual(network_data.teamslaves, value)

        for value in teamslaves_values:
            nd = self.assert_parse("network --device team0 --bootproto=dhcp")
            nd.teamslaves = value
            s = "%s" % nd
            nd2 = self.assert_parse(s)
            self.assertEqual(value, nd2.teamslaves)

class F21_TestCase(F20_TestCase):
    def runTest(self):
        F20_TestCase.runTest(self)

        self.assert_parse("network --device=eth0 --interfacename=BLAH",
                          "network  --bootproto=dhcp --device=eth0 --interfacename=BLAH\n")

class F22_TestCase(F21_TestCase):
    def runTest(self):
        F21_TestCase.runTest(self)

        # bridge options
        # pass
        self.assert_parse("network --device bridge0 --bootproto dhcp --bridgeslaves=ens3,ens7 --bridgeopts=priority=40000",
                          "network  --bootproto=dhcp --device=bridge0 --bridgeslaves=ens3,ens7 --bridgeopts=priority=40000\n")
        self.assert_parse("network --device bridge0 --bootproto dhcp "
                          "--bridgeslaves=ens3,ens7 "
                          "--bridgeopts=priority=40000,hello-time=3")
        # fail
        # slaves missing
        self.assert_parse_error("network --device bridge0 --bootproto dhcp "
                                "--bridgeopts=priority=40000")
        # bad options format
        self.assert_parse_error("network --device bridge0 --bootproto dhcp "
                                "--bridgeslaves=ens3,ens7 "
                                '--bridgeopts="priority=40000 hello-time=3"')
        self.assert_parse_error("network --device bridge0 --bootproto dhcp "
                                "--bridgeslaves=ens3,ens7 "
                                "--bridgeopts=priority")

class F24_TestCase(F22_TestCase):
    def runTest(self):
        F22_TestCase.runTest(self)

        # Test ipv4 only settings
        cmd = "network --noipv4 --activate --hostname=blah.test.com --ipv6=1:2:3:4:5:6:7:8 --device eth0 --nameserver=1:1:1:1::,2:2:2:2::"
        nd = self.assert_parse(cmd)
        self.assertEqual(nd.bootProto, "")
        self.assertEqual(nd.hostname, "blah.test.com")
        self.assertTrue(nd.noipv4)
        self.assertEqual(nd.ipv6, "1:2:3:4:5:6:7:8")
        self.assertIn("1:1:1:1::", nd.nameserver)
        self.assertIn("2:2:2:2::", nd.nameserver)
        self.assertEqual(nd.device, "eth0")

class F25_TestCase(F24_TestCase):
    def runTest(self):
        F24_TestCase.runTest(self)

        # activating a device
        network_data = self.assert_parse("network --device eth0")
        self.assertEqual(network_data.activate, None)
        network_data = self.assert_parse("network --device eth0 --no-activate")
        self.assertEqual(network_data.activate, False)
        network_data = self.assert_parse("network --device eth0 --activate")
        self.assertEqual(network_data.activate, True)
        network_data = self.assert_parse("network --device eth0 --activate --no-activate")
        self.assertEqual(network_data.activate, False)
        network_data = self.assert_parse("network --device eth0 --no-activate --activate")
        self.assertEqual(network_data.activate, True)

class F27_TestCase(F25_TestCase):
    def runTest(self):
        F25_TestCase.runTest(self)

        # binding the configuration to mac
        network_data = self.assert_parse("network --device eth0 --bindto mac")
        self.assertEqual(network_data.bindto, BIND_TO_MAC)
        network_data = self.assert_parse("network --device eth0")
        self.assertIsNone(network_data.bindto)
        # not allowed for vlan device type
        vlan_cmd = "network --device ens3 --vlanid 222 --bootproto dhcp"
        self.assert_parse(vlan_cmd)
        self.assert_parse_error(vlan_cmd + " --bindto mac")
        # but allowed for vlan over bond defined by single command, binds bond slaves
        vlan_over_bond_cmd = "network --device bond0 --bootproto static --ip 10.34.39.222 --netmask 255.255.255.0 --gateway 10.34.39.254 --bondslaves=ens4,ens5 --bondopts=mode=active-backup,miimon-100,primary=ens4 --activate --vlanid=222 --activate --onboot=no"
        self.assert_parse(vlan_over_bond_cmd)
        self.assert_parse(vlan_over_bond_cmd + " --bindto mac")

class RHEL7_TestCase(F20_TestCase):
    def runTest(self):
        F20_TestCase.runTest(self)

        # there needs to be a vlan id after a dot & only one dot is allowed
        self.assert_parse_error("network --interfacename=abc.")
        self.assert_parse_error("network --interfacename=abc.def")
        self.assert_parse_error("network --interfacename=abc..")
        self.assert_parse_error("network --interfacename=abc.123.456")
        # 'vlan' can't be followed by a '.'
        self.assert_parse_error("network --interfacename=vlan.123")
        self.assert_parse_error("network --interfacename=vlan.")
        self.assert_parse_error("network --interfacename=vlan..")
        self.assert_parse_error("network --interfacename=vlan.abc")
        self.assert_parse("network --interfacename=abc.123")

        # if the device name begins with 'vlan', vlan id needs to follow
        self.assert_parse_error("network --interfacename=vlan")
        self.assert_parse_error("network --interfacename=vlanabcd")
        # a valid vlan id needs to directly follow the 'vlan' prefix
        self.assert_parse_error("network --interfacename=vlanabcd123")
        self.assert_parse_error("network --interfacename=vlanabcd123zxy")
        self.assert_parse_error("network --interfacename=vlan123zxy")
        self.assert_parse("network --interfacename=vlan123")

        # vlan ids go from 0 to 4095
        self.assert_parse("network --interfacename=vlan0")
        self.assert_parse("network --interfacename=vlan4095")
        self.assert_parse("network --interfacename=abc.0")
        self.assert_parse("network --interfacename=abc.4095")
        self.assert_parse_error("network --interfacename=vlan4096")
        self.assert_parse_error("network --interfacename=vlan9001")
        self.assert_parse_error("network --interfacename=abc.9001")

        # bridge options
        # pass
        self.assert_parse("network --device bridge0 --bootproto dhcp --bridgeslaves=ens3,ens7 --bridgeopts=priority=40000",
                          "network  --bootproto=dhcp --device=bridge0 --bridgeslaves=ens3,ens7 --bridgeopts=priority=40000\n")
        self.assert_parse("network --device bridge0 --bootproto dhcp "
                          "--bridgeslaves=ens3,ens7 "
                          "--bridgeopts=priority=40000,hello-time=3")

        # fail
        # slaves missing
        self.assert_parse_error("network --device bridge0 --bootproto dhcp "
                                "--bridgeopts=priority=40000")
        # bad options format
        self.assert_parse_error("network --device bridge0 --bootproto dhcp "
                                "--bridgeslaves=ens3,ens7 "
                                '--bridgeopts="priority=40000 hello-time=3"')
        self.assert_parse_error("network --device bridge0 --bootproto dhcp "
                                "--bridgeslaves=ens3,ens7 "
                                "--bridgeopts=priority")

        # activating a device
        network_data = self.assert_parse("network --device eth0")
        self.assertEqual(network_data.activate, None)
        network_data = self.assert_parse("network --device eth0 --no-activate")
        self.assertEqual(network_data.activate, False)
        network_data = self.assert_parse("network --device eth0 --activate")
        self.assertEqual(network_data.activate, True)
        network_data = self.assert_parse("network --device eth0 --activate --no-activate")
        self.assertEqual(network_data.activate, False)
        network_data = self.assert_parse("network --device eth0 --no-activate --activate")
        self.assertEqual(network_data.activate, True)

        # binding the configuration to mac
        network_data = self.assert_parse("network --device eth0 --bindto mac")
        self.assertEqual(network_data.bindto, BIND_TO_MAC)
        network_data = self.assert_parse("network --device eth0")
        self.assertIsNone(network_data.bindto)
        # not allowed for vlan device type
        vlan_cmd = "network --device ens3 --vlanid 222 --bootproto dhcp"
        self.assert_parse(vlan_cmd)
        self.assert_parse_error(vlan_cmd + " --bindto mac")
        # but allowed for vlan over bond defined by single command, binds bond slaves
        vlan_over_bond_cmd = "network --device bond0 --bootproto static --ip 10.34.39.222 --netmask 255.255.255.0 --gateway 10.34.39.254 --bondslaves=ens4,ens5 --bondopts=mode=active-backup,miimon-100,primary=ens4 --activate --vlanid=222 --activate --onboot=no"
        self.assert_parse(vlan_over_bond_cmd)
        self.assert_parse(vlan_over_bond_cmd + " --bindto mac")

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, F19
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser, commaSplit

from pykickstart.i18n import _

class FC3_Lang(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()
        self.lang = kwargs.get("lang", "")

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.lang:
            retval += "# System language\nlang %s\n" % self.lang

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="lang", description="""
            This required command sets the language to use during installation
            and the default language to use on the installed system to ``<id>``.
            This can be the same as any recognized setting for the ``$LANG``
            environment variable, though not all languages are supported during
            installation.

            Certain languages (mainly Chinese, Japanese, Korean, and Indic
            languages) are not supported during text mode installation. If one
            of these languages is specified using the lang command, installation
            will continue in English though the running system will have the
            specified langauge by default.

            The file ``/usr/share/system-config-language/locale-list`` provides a
            list the valid language codes in the first column of each line and
            is part of the system-config-languages package.""", version=FC3)
        op.add_argument("lang", metavar="<lang>", nargs=1, version=FC3,
                        help="Language ID.")
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)
        assert len(ns.lang) == 1

        if extra:
            mapping = {"command": "lang", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        self.set_to_self(ns)
        self.lang = ns.lang[0]
        return self

class F19_Lang(FC3_Lang):
    removedKeywords = FC3_Lang.removedKeywords
    removedAttrs = FC3_Lang.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        FC3_Lang.__init__(self, writePriority, *args, **kwargs)
        self.addsupport = kwargs.get("addsupport", [])

        self.op = self._getParser()

    def __str__(self):
        s = FC3_Lang.__str__(self)
        if s and self.addsupport:
            s = s.rstrip()
            s += " --addsupport=%s\n" % ",".join(self.addsupport)
        return s

    def _getParser(self):
        op = FC3_Lang._getParser(self)
        op.add_argument("--addsupport", type=commaSplit,
                        metavar="LOCALE", help="""
                        Install the support packages for the given locales,
                        specified as a comma-separated list. Each locale may be
                        specified in the same ways as the primary language may
                        be, as described above.""", version=F19)
        return op



from tools import ksverdiff
import unittest.mock as mock
from unittest.mock import call
from unittest import TestCase
from pykickstart.version import versionMap


class ListVersions_TestCase(TestCase):
    @mock.patch('tools.ksverdiff.print')
    def runTest(self, _print):
        retval = ksverdiff.main(['-l'])
        self.assertEqual(retval, 0)

        _print_calls = []
        for key in sorted(versionMap.keys()):
            _print_calls.append(call(key))
        _print.assert_has_calls(_print_calls)

class MissingFromToOptions_TestCase(TestCase):
    @mock.patch('tools.ksverdiff.print')
    def runTest(self, _print):
        retval = ksverdiff.main([])
        self.assertEqual(retval, 1)
        _print.assert_called_with('You must specify two syntax versions.')

class InvalidKSVersion_TestCase(TestCase):
    @mock.patch('tools.ksverdiff.print')
    def runTest(self, _print):
        retval = ksverdiff.main(['-f', 'F23', '-t', 'INVALID'])
        self.assertEqual(retval, 1)
        _print.assert_called_with('The version Unsupported version specified: INVALID is not supported by pykickstart')

class From23To25_TestCase(TestCase):
    @mock.patch('tools.ksverdiff.print')
    def runTest(self, _print):
        retval = ksverdiff.main(['-f', 'F23', '-t', 'F25'])
        self.assertEqual(retval, 0)
        _calls = []
        _calls.append(call('The following commands were removed in F25:'))
        _calls.append(call(''))
        _calls.append(call('The following commands were deprecated in F25:'))
        _calls.append(call('device dmraid multipath upgrade'))
        _calls.append(call())
        _calls.append(call('The following commands were added in F25:'))
        _calls.append(call(''))
        _calls.append(call('The following options were added to the network command in F25:'))
        _calls.append(call('--no-activate'))
        _calls.append(call())
        _calls.append(call('The following options were added to the raid command in F25:'))
        _calls.append(call('--chunksize'))
        _calls.append(call())
        _calls.append(call('The following options were added to the sshpw command in F25:'))
        _calls.append(call('--sshkey'))

        _print.assert_has_calls(_calls)

class From10To11_Deprecated_TestCase(TestCase):
    @mock.patch('tools.ksverdiff.print')
    def runTest(self, _print):
        retval = ksverdiff.main(['-f', 'F10', '-t', 'F11'])
        self.assertEqual(retval, 0)
        _calls = []
        # validate only deprecation calls
        _calls.append(call('The following options were deprecated from the xconfig command in F11:'))
        _calls.append(call('--depth --driver --resolution --videoram'))
        _print.assert_has_calls(_calls)

class From10To14_Removed_TestCase(TestCase):
    @mock.patch('tools.ksverdiff.print')
    def runTest(self, _print):
        retval = ksverdiff.main(['-f', 'F10', '-t', 'F14'])
        self.assertEqual(retval, 0)
        _calls = []
        # validate only removed calls
        _calls.append(call('The following options were removed from the xconfig command in F14:'))
        _calls.append(call('--depth --driver --resolution --videoram'))
        _print.assert_has_calls(_calls)



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import warnings
from pykickstart.version import FC3, FC6, F18
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError, KickstartDeprecationWarning
from pykickstart.options import KSOptionParser, commaSplit

from pykickstart.i18n import _

class FC3_Timezone(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.isUtc = kwargs.get("isUtc", False)
        self.timezone = kwargs.get("timezone", "")

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.timezone:
            if self.isUtc:
                utc = "--utc"
            else:
                utc = ""

            retval += "# System timezone\ntimezone %s %s\n" % (utc, self.timezone)

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="timezone", description="""
                            This required command sets the system time zone to
                            which may be any of the time zones listed by
                            timeconfig.""", version=FC3)
        op.add_argument("--utc", dest="isUtc", action="store_true",
                        default=False, version=FC3, help="""
                        If present, the system assumes the hardware clock is set
                        to UTC (Greenwich Mean) time.

                       *To get the list of supported timezones, you can either
                        run this script:
                        http://vpodzime.fedorapeople.org/timezones_list.py or
                        look at this list:
                        http://vpodzime.fedorapeople.org/timezones_list.txt*
                        """)
        op.add_argument("timezone", metavar="<timezone>", nargs=1,
                        version=FC3, help="""
                        Timezone name, e.g. Europe/Sofia.""")
        return op

    # Caution: This method is overridden in the RHEL7_Timezone & F25_Timezone classes
    # by a new implementation not calling this method.
    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        assert len(ns.timezone) == 1

        if extra:
            mapping = {"command": "timezone", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        self.timezone = ns.timezone[0]
        return self

class FC6_Timezone(FC3_Timezone):
    removedKeywords = FC3_Timezone.removedKeywords
    removedAttrs = FC3_Timezone.removedAttrs

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.timezone:
            if self.isUtc:
                utc = "--isUtc"
            else:
                utc = ""

            retval += "# System timezone\ntimezone %s %s\n" % (utc, self.timezone)

        return retval

    def _getParser(self):
        op = FC3_Timezone._getParser(self)
        op.add_argument("--utc", "--isUtc", dest="isUtc", action="store_true",
                        default=False, version=FC6, help="""
                        The ``--isUtc`` option was added.""")
        return op

class F18_Timezone(FC6_Timezone):
    def __init__(self, writePriority=0, *args, **kwargs):
        FC6_Timezone.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()
        self.nontp = kwargs.get("nontp", False)
        self.ntpservers = kwargs.get("ntpservers", set())

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.timezone:
            retval += "# System timezone\n"
            retval += "timezone " + self._getArgsAsStr() + "\n"

        return retval

    def _getArgsAsStr(self):
        retval = self.timezone

        if self.isUtc:
            retval += " --isUtc"

        if self.nontp:
            retval += " --nontp"

        if self.ntpservers:
            retval += " --ntpservers=" + ",".join(self.ntpservers)

        return retval

    def _getParser(self):
        op = FC6_Timezone._getParser(self)
        op.add_argument("--nontp", action="store_true", default=False,
                        version=F18, help="""
                        Disable automatic starting of NTP service.

                        ``--nontp`` and ``--ntpservers`` are mutually exclusive.
                        """)
        op.add_argument("--ntpservers", dest="ntpservers", type=commaSplit,
                        metavar="<server1>,<server2>,...,<serverN>",
                        version=F18, help="""
                        Specify a list of NTP servers to be used (comma-separated
                        list with no spaces). The chrony package is automatically
                        installed when this option is used. If you don't want the
                        package to be automatically installed then use ``-chrony``
                        in package selection. For example::

                        ``timezone --ntpservers=ntp.cesnet.cz,tik.nic.cz Europe/Prague``
                        """)
        return op

    # Caution: The parse() method is overridden in the RHEL7_Timezone & F25_Timezone classes
    # by a new implementation not calling this method.
    def parse(self, args):
        FC6_Timezone.parse(self, args)

        # NTP servers can only be listed once in F18, so weed out the duplicates now.
        self.ntpservers = list(set(self.ntpservers))

        if self.ntpservers and self.nontp:
            msg = _("Options --nontp and --ntpservers are mutually exclusive")
            raise KickstartParseError(msg, lineno=self.lineno)

        return self

class F23_Timezone(F18_Timezone):
    def __init__(self, *args, **kwargs):
        F18_Timezone.__init__(self, *args, **kwargs)
        self.ntpservers = kwargs.get("ntpservers", list())

    # Caution: This method is overridden in the RHEL7_Timezone & F25_Timezone classes
    # by a new implementation not calling this method.
    def parse(self, args):
        FC6_Timezone.parse(self, args)

        if self.ntpservers and self.nontp:
            msg = _("Options --nontp and --ntpservers are mutually exclusive")
            raise KickstartParseError(msg, lineno=self.lineno)

        return self

# About the RHEL7_Timezone & F25_Timezone command classes
# =======================================================
#
# The F18_Timezone command class used in RHEL <=7.2 the F23_Timezone
# command class used in Fedora <=24 the always required a timezone
# specification to be provided.
#
# On the other hand the timezone command has also some commands that don't
# really need a timezone to be specified to work, like "--utc/--isutc".
#
# So if you for example wanted to set the hwclock to the UTC mode but still
# wanted the user to set a timezone manually in the UI (by not providing a
# timezone specification), you were out of luck - a timezone parsing error
# would be raised.
#
# To fix this we need to remove the requirement on always providing one
# argument to the timezone command. As the requirement is present
# quite deep in the class hierarchy (in the original FC3_Timezone class)
# we reimplement the parse() method in the RHEL7_Timezone and F25_Timezone
# classes and avoid calling the ancestors parse(), as it otherwise done in all
# other command child classes.

class RHEL7_Timezone(F18_Timezone):
    def __init__(self, writePriority=0, *args, **kwargs):
        F18_Timezone.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

    def _getParser(self):
        op = KSOptionParser(prog="timezone", description="""
                            This required command sets the system time zone to
                            which may be any of the time zones listed by
                            timeconfig.""", version=FC3)
        op.add_argument("timezone", metavar="<timezone>", nargs="*",
                        version=FC3, help="""
                        Timezone name, e.g. Europe/Sofia.
                        This is optional but at least one of the options needs
                        to be used if no timezone is specified.
                        """)
        op.add_argument("--utc", "--isUtc", dest="isUtc", action="store_true",
                        default=False, version=FC6, help="""
                        If present, the system assumes the hardware clock is set
                        to UTC (Greenwich Mean) time.

                       *To get the list of supported timezones, you can either
                        run this script:
                        http://vpodzime.fedorapeople.org/timezones_list.py or
                        look at this list:
                        http://vpodzime.fedorapeople.org/timezones_list.txt*
                        """)
        op.add_argument("--nontp", action="store_true", default=False,
                        version=F18, help="""
                        Disable automatic starting of NTP service.

                        ``--nontp`` and ``--ntpservers`` are mutually exclusive.
                        """)
        op.add_argument("--ntpservers", dest="ntpservers", type=commaSplit,
                        metavar="<server1>,<server2>,...,<serverN>",
                        version=F18, help="""
                        Specify a list of NTP servers to be used (comma-separated
                        list with no spaces). The chrony package is automatically
                        installed when this option is used. If you don't want the
                        package to be automatically installed then use ``-chrony``
                        in package selection. For example::

                        ``timezone --ntpservers=ntp.cesnet.cz,tik.nic.cz Europe/Prague``
                        """)
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)

        # just "timezone" without any arguments and timezone specification doesn't really make sense,
        # so throw an error when we see it (it might even be an indication of an incorrect machine generated kickstart)
        if not args:
            error_message = _("At least one option and/or an argument are expected for the %s command") % "timezone"
            raise KickstartParseError(error_message, lineno=self.lineno)

        # To be able to support the timezone command being used without
        # a timezone specification:
        # - we don't call the parse() method of the ancestors
        # -> due to the FC3 parse() method that would be eventually called,
        #    which throws an exception if no timezone specification is provided
        # - we implement the relevant functionality of the ancestor methods here

        if len(ns.timezone) == 1:
            self.timezone = ns.timezone[0]
        elif len(ns.timezone) > 1:
            error_message = _("One or zero arguments are expected for the %s command") % "timezone"
            raise KickstartParseError(error_message, lineno=self.lineno)

        if self.ntpservers and self.nontp:
            msg = _("Options --nontp and --ntpservers are mutually exclusive")
            raise KickstartParseError(msg, lineno=self.lineno)

        return self

class F25_Timezone(F23_Timezone):
    def __init__(self, writePriority=0, *args, **kwargs):
        F23_Timezone.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

    def __str__(self):
        retval = KickstartCommand.__str__(self)
        args = self._getArgsAsStr()
        if args:
            retval += "# System timezone\n"
            retval += "timezone" + args + "\n"

        return retval

    def _getArgsAsStr(self):
        retval = ""

        if self.timezone:
            retval += " " + self.timezone

        if self.isUtc:
            retval += " --isUtc"

        if self.nontp:
            retval += " --nontp"

        if self.ntpservers:
            retval += " --ntpservers=" + ",".join(self.ntpservers)

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="timezone", description="""
                            This required command sets the system time zone to
                            which may be any of the time zones listed by
                            timeconfig.""", version=FC3)
        op.add_argument("timezone", metavar="<timezone>", nargs="*",
                        version=FC3, help="""
                        Timezone name, e.g. Europe/Sofia.
                        This is optional but at least one of the options needs
                        to be used if no timezone is specified.
                        """)
        op.add_argument("--utc", "--isUtc", dest="isUtc", action="store_true",
                        default=False, version=FC6, help="""
                        If present, the system assumes the hardware clock is set
                        to UTC (Greenwich Mean) time.

                       *To get the list of supported timezones, you can either
                        run this script:
                        http://vpodzime.fedorapeople.org/timezones_list.py or
                        look at this list:
                        http://vpodzime.fedorapeople.org/timezones_list.txt*
                        """)
        op.add_argument("--nontp", action="store_true", default=False,
                        version=F18, help="""
                        Disable automatic starting of NTP service.

                        ``--nontp`` and ``--ntpservers`` are mutually exclusive.
                        """)
        op.add_argument("--ntpservers", dest="ntpservers", type=commaSplit,
                        metavar="<server1>,<server2>,...,<serverN>",
                        version=F18, help="""
                        Specify a list of NTP servers to be used (comma-separated
                        list with no spaces). The chrony package is automatically
                        installed when this option is used. If you don't want the
                        package to be automatically installed then use ``-chrony``
                        in package selection. For example::

                        ``timezone --ntpservers=ntp.cesnet.cz,tik.nic.cz Europe/Prague``
                        """)
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)

        # just "timezone" without any arguments and timezone specification doesn't really make sense,
        # so throw an error when we see it (it might even be an indication of an incorrect machine generated kickstart)
        if not args:
            error_message = _("At least one option and/or an argument are expected for the %s command") % "timezone"
            raise KickstartParseError(error_message, lineno=self.lineno)

        # To be able to support the timezone command being used without
        # a timezone specification:
        # - we don't call the parse() method of the ancestors
        # -> due to the FC3 parse() method that would be eventually called,
        #    which throws an exception if no timezone specification is provided
        # - we implement the relevant functionality of the ancestor methods here

        if len(ns.timezone) == 1:
            self.timezone = ns.timezone[0]
        elif len(ns.timezone) > 1:
            error_message = _("One or zero arguments are expected for the %s command") % "timezone"
            raise KickstartParseError(error_message, lineno=self.lineno)

        if self.ntpservers and self.nontp:
            msg = _("Options --nontp and --ntpservers are mutually exclusive")
            raise KickstartParseError(msg, lineno=self.lineno)

        return self

class F32_Timezone(F25_Timezone):

    def __init__(self, writePriority=0, *args, **kwargs):
        F25_Timezone.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

    def _getArgsAsStr(self):
        retval = ""

        if self.timezone:
            retval += " " + self.timezone

        if self.isUtc:
            retval += " --utc"

        if self.nontp:
            retval += " --nontp"

        if self.ntpservers:
            retval += " --ntpservers=" + ",".join(self.ntpservers)

        return retval

    def parse(self, args):
        F25_Timezone.parse(self, args)

        if "--isUtc" in args:
            warnings.warn(_("The option --isUtc will be deprecated in future releases. Please "
                            "modify your kickstart file to replace this option with its preferred "
                            "alias --utc."),
                          KickstartDeprecationWarning)

        return self


class F33_Timezone(F32_Timezone):

    def __init__(self, writePriority=0, *args, **kwargs):
        F32_Timezone.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

    def parse(self, args):
        F32_Timezone.parse(self, args)

        if self.ntpservers:
            warnings.warn(_("The option --ntpservers will be deprecated in future releases. Please "
                            "modify your kickstart file to replace this option with "
                            "timesource --ntp-server <server hostname> command invocation, "
                            "one per NTP server."),
                          KickstartDeprecationWarning)
        if self.nontp:
            warnings.warn(_("The option --nontp will be deprecated in future releases. Please "
                            "modify your kickstart file to replace this option with "
                            "timesource --ntp-disable command invocation."),
                          KickstartDeprecationWarning)
        return self



#
# Vendula Poncova <vponcova@redhat.com>
#
# Copyright 2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.base import KickstartCommand
from pykickstart.options import KSOptionParser
from pykickstart.version import F28


class F28_Authselect(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.authselect = kwargs.get("authselect", "")

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.authselect:
            retval += "# System authorization information\nauthselect %s\n" % self.authselect

        return retval

    def parse(self, args):
        self.authselect = self.currentLine[len(self.currentCmd):].strip()
        return self

    def _getParser(self):
        op = KSOptionParser(prog="authselect",  description="""
                            This command sets up the authentication options
                            for the system. This is just a wrapper around the
                            authselect program, so all options recognized by
                            that program are valid for this command. See the
                            manual page for authselect for a complete list.""",
                            version=F28)

        op.add_argument("options", metavar="[options]", help="""
                        See ``man authselect``.""", version=F28)
        return op



#
# Vendula Poncova <vponcova@redhat.com>
#
# Copyright 2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
__all__ = ["F29Handler"]

from pykickstart import commands
from pykickstart.base import BaseHandler
from pykickstart.version import F29

class F29Handler(BaseHandler):
    version = F29

    commandMap = {
        "auth": commands.authconfig.F28_Authconfig,
        "authconfig": commands.authconfig.F28_Authconfig,
        "authselect": commands.authselect.F28_Authselect,
        "autopart": commands.autopart.F29_AutoPart,
        "autostep": commands.autostep.FC3_AutoStep,
        "bootloader": commands.bootloader.F29_Bootloader,
        "btrfs": commands.btrfs.F23_BTRFS,
        "cdrom": commands.cdrom.FC3_Cdrom,
        "clearpart": commands.clearpart.F28_ClearPart,
        "cmdline": commands.displaymode.F26_DisplayMode,
        "device": commands.device.F24_Device,
        "deviceprobe": commands.deviceprobe.F29_DeviceProbe,
        "dmraid": commands.dmraid.F24_DmRaid,
        "driverdisk": commands.driverdisk.F14_DriverDisk,
        "module" : commands.module.F29_Module,
        "eula": commands.eula.F20_Eula,
        "fcoe": commands.fcoe.F28_Fcoe,
        "firewall": commands.firewall.F28_Firewall,
        "firstboot": commands.firstboot.FC3_Firstboot,
        "graphical": commands.displaymode.F26_DisplayMode,
        "group": commands.group.F12_Group,
        "halt": commands.reboot.F23_Reboot,
        "harddrive": commands.harddrive.FC3_HardDrive,
        "hmc" : commands.hmc.F28_Hmc,
        "ignoredisk": commands.ignoredisk.F29_IgnoreDisk,
        "install": commands.install.F29_Install,
        "iscsi": commands.iscsi.F17_Iscsi,
        "iscsiname": commands.iscsiname.FC6_IscsiName,
        "keyboard": commands.keyboard.F18_Keyboard,
        "lang": commands.lang.F19_Lang,
        "liveimg": commands.liveimg.F19_Liveimg,
        "logging": commands.logging.FC6_Logging,
        "logvol": commands.logvol.F29_LogVol,
        "mediacheck": commands.mediacheck.FC4_MediaCheck,
        "method": commands.method.F28_Method,
        "mount": commands.mount.F27_Mount,
        "multipath": commands.multipath.F24_MultiPath,
        "network": commands.network.F27_Network,
        "nfs": commands.nfs.FC6_NFS,
        "nvdimm": commands.nvdimm.F28_Nvdimm,
        "ostreesetup": commands.ostreesetup.F21_OSTreeSetup,
        "part": commands.partition.F29_Partition,
        "partition": commands.partition.F29_Partition,
        "poweroff": commands.reboot.F23_Reboot,
        "raid": commands.raid.F29_Raid,
        "realm": commands.realm.F19_Realm,
        "reboot": commands.reboot.F23_Reboot,
        "repo": commands.repo.F27_Repo,
        "reqpart": commands.reqpart.F23_ReqPart,
        "rescue": commands.rescue.F10_Rescue,
        "rootpw": commands.rootpw.F18_RootPw,
        "selinux": commands.selinux.FC3_SELinux,
        "services": commands.services.FC6_Services,
        "shutdown": commands.reboot.F23_Reboot,
        "skipx": commands.skipx.FC3_SkipX,
        "snapshot": commands.snapshot.F26_Snapshot,
        "sshpw": commands.sshpw.F24_SshPw,
        "sshkey": commands.sshkey.F22_SshKey,
        "text": commands.displaymode.F26_DisplayMode,
        "timezone": commands.timezone.F25_Timezone,
        "updates": commands.updates.F7_Updates,
        "upgrade": commands.upgrade.F29_Upgrade,  # RemovedCommand
        "url": commands.url.F27_Url,
        "user": commands.user.F24_User,
        "vnc": commands.vnc.F9_Vnc,
        "volgroup": commands.volgroup.F21_VolGroup,
        "xconfig": commands.xconfig.F14_XConfig,
        "zerombr": commands.zerombr.F9_ZeroMbr,
        "zfcp": commands.zfcp.F14_ZFCP,
    }

    dataMap = {
        "BTRFSData": commands.btrfs.F23_BTRFSData,
        "DriverDiskData": commands.driverdisk.F14_DriverDiskData,
        "DeviceData": commands.device.F8_DeviceData,
        "DmRaidData": commands.dmraid.FC6_DmRaidData,
        "ModuleData" : commands.module.F29_ModuleData,
        "FcoeData": commands.fcoe.F28_FcoeData,
        "GroupData": commands.group.F12_GroupData,
        "IscsiData": commands.iscsi.F17_IscsiData,
        "LogVolData": commands.logvol.F29_LogVolData,
        "MountData": commands.mount.F27_MountData,
        "MultiPathData": commands.multipath.FC6_MultiPathData,
        "NetworkData": commands.network.F27_NetworkData,
        "NvdimmData": commands.nvdimm.F28_NvdimmData,
        "PartData": commands.partition.F29_PartData,
        "RaidData": commands.raid.F29_RaidData,
        "RepoData": commands.repo.F27_RepoData,
        "SnapshotData": commands.snapshot.F26_SnapshotData,
        "SshPwData": commands.sshpw.F24_SshPwData,
        "SshKeyData": commands.sshkey.F22_SshKeyData,
        "UserData": commands.user.F19_UserData,
        "VolGroupData": commands.volgroup.F21_VolGroupData,
        "ZFCPData": commands.zfcp.F14_ZFCPData,
    }



#
# parser.py:  Kickstart file parser.
#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005-2016 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
"""
Main kickstart file processing module.

This module exports several important classes:

    Script - Representation of a single %pre, %pre-install, %post, or %traceback script.

    Packages - Representation of the %packages section.

    KickstartParser - The kickstart file parser state machine.
"""

from __future__ import print_function

try:
    from collections.abc import Iterator
except ImportError:  # python2 compatibility
    from collections import Iterator

import os
import six
import shlex
import sys
import warnings

from pykickstart import constants, version
from pykickstart.errors import KickstartError, KickstartParseError, KickstartParseWarning
from pykickstart.ko import KickstartObject
from pykickstart.load import load_to_str
from pykickstart.options import KSOptionParser
from pykickstart.sections import PackageSection, PreScriptSection, PreInstallScriptSection, \
                                 PostScriptSection, TracebackScriptSection, OnErrorScriptSection, \
                                 NullSection

from pykickstart.i18n import _

STATE_END = "end"
STATE_COMMANDS = "commands"

def _preprocessStateMachine(lineIter):
    l = None
    lineno = 0
    retval = ""

    if six.PY3:
        retval = retval.encode(sys.getdefaultencoding())

    while True:
        try:
            l = next(lineIter)
        except StopIteration:
            break

        # At the end of the file?
        if l == "":
            break

        lineno += 1
        ksurl = None

        ll = l.strip()
        if not ll.startswith("%ksappend"):
            if six.PY3:
                l = l.encode(sys.getdefaultencoding())
            retval += l
            continue

        # Try to pull down the remote file.
        try:
            ksurl = ll.split(' ')[1]
        except:
            raise KickstartParseError(_("Illegal url for %%ksappend: %s") % ll, lineno=lineno)

        try:
            contents = load_to_str(ksurl)
        except KickstartError as e:
            raise KickstartError(_("Unable to open %%ksappend file: %s") % str(e), lineno=lineno)

        # If that worked, write the remote file to the output kickstart
        # file in one burst.  This allows multiple %ksappend lines to
        # exist.
        if contents is not None:
            retval += contents.encode(sys.getdefaultencoding())

    return retval

def preprocessFromStringToString(s):
    """Preprocess the kickstart file, provided as the string s.  This
       method is currently only useful for handling %ksappend lines, which
       need to be fetched before the real kickstart parser can be run.
       Returns the complete kickstart file as a string.
    """
    i = iter(s.splitlines(True) + [""])
    return _preprocessStateMachine(i)

def preprocessKickstartToString(f):
    """Preprocess the kickstart file, given by the filename f.  This
       method is currently only useful for handling %ksappend lines,
       which need to be fetched before the real kickstart parser can be
       run.  Returns the complete kickstart file as a string.
    """
    try:
        contents = load_to_str(f)
    except KickstartError as e:
        raise KickstartError(_("Unable to open input kickstart file: %s") % str(e), lineno=0)

    return _preprocessStateMachine(iter(contents.splitlines(True)))

def preprocessFromString(s):
    """Preprocess the kickstart file, provided as the string s.  This
       method is currently only useful for handling %ksappend lines,
       which need to be fetched before the real kickstart parser can be
       run.  Returns the location of the complete kickstart file.
    """
    s = preprocessFromStringToString(s)
    if s:
        import tempfile
        (outF, outName) = tempfile.mkstemp(suffix="-ks.cfg")

        os.write(outF, s)
        os.close(outF)
        return outName

    return None

def preprocessKickstart(f):
    """Preprocess the kickstart file, given by the filename f.  This
       method is currently only useful for handling %ksappend lines,
       which need to be fetched before the real kickstart parser can be
       run.  Returns the location of the complete kickstart file.
    """
    s = preprocessKickstartToString(f)
    if s:
        import tempfile
        (outF, outName) = tempfile.mkstemp(suffix="-ks.cfg")

        os.write(outF, s)
        os.close(outF)
        return outName

    return None

class PutBackIterator(Iterator):
    def __init__(self, iterable):
        self._iterable = iter(iterable)
        self._buf = None

    def __iter__(self):
        return self

    def put(self, s):
        self._buf = s

    def next(self):
        if self._buf:
            retval = self._buf
            self._buf = None
            return retval
        else:
            return next(self._iterable)

    def __next__(self):
        return self.next()                          # pylint: disable=not-callable

###
### SCRIPT HANDLING
###
class Script(KickstartObject):
    _ver = version.DEVEL

    """A class representing a single kickstart script.  If functionality beyond
       just a data representation is needed (for example, a run method in
       anaconda), Script may be subclassed.  Although a run method is not
       provided, most of the attributes of Script have to do with running the
       script.  Instances of Script are held in a list by the Version object.
    """
    def __init__(self, script, *args, **kwargs):
        """Create a new Script instance.  Instance attributes:

           :keyword errorOnFail: If execution of the script fails, should anaconda
                                 stop, display an error, and then reboot without
                                 running any other scripts?

           :keyword inChroot: Does the script execute in anaconda's chroot
                              environment or not?

           :keyword interp: The program that should be used to interpret this
                            script.

           :keyword lineno: The line number this script starts on.

           :keyword logfile: Where all messages from the script should be logged.

           :keyword script: A string containing all the lines of the script.

           :keyword type: The type of the script, which can be KS_SCRIPT_* from
                          :mod:`pykickstart.constants`.
        """
        KickstartObject.__init__(self, *args, **kwargs)
        self.script = "".join(script)

        self.interp = kwargs.get("interp", "/bin/sh")
        self.inChroot = kwargs.get("inChroot", False)
        self.lineno = kwargs.get("lineno", None)
        self.logfile = kwargs.get("logfile", None)
        self.errorOnFail = kwargs.get("errorOnFail", False)
        self.type = kwargs.get("type", constants.KS_SCRIPT_PRE)

    def __str__(self):
        """Return a string formatted for output to a kickstart file."""
        retval = ""

        if self.type == constants.KS_SCRIPT_PRE:
            retval += '\n%pre'
        elif self.type == constants.KS_SCRIPT_POST:
            retval += '\n%post'
        elif self.type == constants.KS_SCRIPT_TRACEBACK:
            retval += '\n%traceback'
        elif self.type == constants.KS_SCRIPT_PREINSTALL:
            retval += '\n%pre-install'
        elif self.type == constants.KS_SCRIPT_ONERROR:
            retval += '\n%onerror'

        if self.interp != "/bin/sh" and self.interp:
            retval += " --interpreter=%s" % self.interp
        if self.type == constants.KS_SCRIPT_POST and not self.inChroot:
            retval += " --nochroot"
        if self.logfile is not None:
            retval += " --logfile=%s" % self.logfile
        if self.errorOnFail:
            retval += " --erroronfail"

        if self.script.endswith("\n"):
            if self._ver >= version.F8:
                return retval + "\n%s%%end\n" % self.script
            else:
                return retval + "\n%s" % self.script
        else:
            if self._ver >= version.F8:
                return retval + "\n%s\n%%end\n" % self.script
            else:
                return retval + "\n%s\n" % self.script

##
## PACKAGE HANDLING
##
class Group(KickstartObject):
    """A class representing a single group in the %packages section."""
    def __init__(self, name="", include=constants.GROUP_DEFAULT):
        """Create a new Group instance.  Instance attributes:

           name    -- The group's identifier
           include -- The level of how much of the group should be included.
                      Values can be GROUP_* from pykickstart.constants.
        """
        KickstartObject.__init__(self)
        self.name = name
        self.include = include

    def __str__(self):
        """Return a string formatted for output to a kickstart file."""
        if self.include == constants.GROUP_REQUIRED:
            return "@%s --nodefaults" % self.name
        elif self.include == constants.GROUP_ALL:
            return "@%s --optional" % self.name
        else:
            return "@%s" % self.name

    def __lt__(self, other):
        return self.name < other.name

    def __le__(self, other):
        return self.name <= other.name

    def __eq__(self, other):
        return self.name == other.name

    def __ne__(self, other):
        return self.name != other.name

    def __gt__(self, other):
        return self.name > other.name

    def __ge__(self, other):
        return self.name >= other.name

    __hash__ = KickstartObject.__hash__

class Packages(KickstartObject):
    _ver = version.DEVEL

    """A class representing the %packages section of the kickstart file."""
    def __init__(self, *args, **kwargs):
        """Create a new Packages instance.  Instance attributes:

           addBase       -- Should the Base group be installed even if it is
                            not specified?
           nocore        -- Should the Core group be skipped?  This results in
                            a %packages section that basically only installs the
                            packages you list, and may not be a usable system.
           default       -- Should the default package set be selected?
           environment   -- What base environment should be selected?  Only one
                            may be chosen at a time.
           excludedList  -- A list of all the packages marked for exclusion in
                            the %packages section, without the leading minus
                            symbol.
           excludeDocs   -- Should documentation in each package be excluded?
           groupList     -- A list of Group objects representing all the groups
                            specified in the %packages section.  Names will be
                            stripped of the leading @ symbol.
           excludedGroupList -- A list of Group objects representing all the
                                groups specified for removal in the %packages
                                section.  Names will be stripped of the leading
                                -@ symbols.
           handleMissing -- If unknown packages are specified in the %packages
                            section, should it be ignored or not?  Values can
                            be KS_MISSING_* from pykickstart.constants.
           handleBroken  -- If packages with conflicts are specified in the
                            %packages section, should it be ignored or not?
                            Values can be KS_BROKEN_* from pykickstart.constants.
           packageList   -- A list of all the packages specified in the
                            %packages section.
           instLangs     -- A list of languages to install.
           multiLib      -- Whether to use yum's "all" multilib policy.
           excludeWeakdeps -- Whether to exclude weak dependencies.
           timeout       -- Number of seconds to wait for a connection before
                            yum's or dnf's timing out or None.
           retries       -- Number of times yum's or dnf's attempt to retrieve
                            a file should retry before returning an error.
           seen          -- If %packages was ever used in the kickstart file,
                            this attribute will be set to True.

        """
        KickstartObject.__init__(self, *args, **kwargs)

        self.addBase = True
        self.nocore = False
        self.default = False
        self.environment = None
        self.excludedList = []
        self.excludedGroupList = []
        self.excludeDocs = False
        self.groupList = []
        self.handleMissing = constants.KS_MISSING_PROMPT
        self.handleBroken = constants.KS_BROKEN_REPORT
        self.packageList = []
        self.instLangs = None
        self.multiLib = False
        self.excludeWeakdeps = False
        self.timeout = None
        self.retries = None
        self.seen = False

    def __str__(self):
        """Return a string formatted for output to a kickstart file."""
        pkgs = self._processPackagesContent()
        retval = ""

        if self.default:
            retval += " --default"
        if self.excludeDocs:
            retval += " --excludedocs"
        if not self.addBase:
            retval += " --nobase"
        if self.nocore:
            retval += " --nocore"
        if self.handleMissing == constants.KS_MISSING_IGNORE:
            retval += " --ignoremissing"
        if self.handleBroken == constants.KS_BROKEN_IGNORE:
            retval += " --ignorebroken"
        if self.instLangs is not None:
            retval += " --inst-langs=%s" % self.instLangs
        if self.multiLib:
            retval += " --multilib"
        if self.excludeWeakdeps:
            retval += " --exclude-weakdeps"
        if self.timeout is not None:
            retval += " --timeout=%d" % self.timeout
        if self.retries is not None:
            retval += " --retries=%d" % self.retries

        if retval == "" and pkgs == "" and not self.seen:
            return ""

        if self._ver >= version.F8:
            return "\n%packages" + retval + "\n" + pkgs + "\n%end\n"
        else:
            return "\n%packages" + retval + "\n" + pkgs + "\n"

    def _processPackagesContent(self):
        pkgs = ""

        if not self.default:
            if self.environment:
                pkgs += "@^%s\n" % self.environment

        grps = self.groupList
        grps.sort()
        for grp in grps:
            pkgs += "%s\n" % grp.__str__()

        p = self.packageList
        p.sort()
        for pkg in p:
            pkgs += "%s\n" % pkg

        grps = self.excludedGroupList
        grps.sort()
        for grp in grps:
            pkgs += "-%s\n" % grp.__str__()

        p = self.excludedList
        p.sort()
        for pkg in p:
            pkgs += "-%s\n" % pkg

        return pkgs

    def _processGroup(self, line):
        op = KSOptionParser(prog="", description="", version=version.DEVEL)
        op.add_argument("--nodefaults", action="store_true", default=False,
                        help="", version=version.DEVEL)
        op.add_argument("--optional", action="store_true", default=False,
                        help="", version=version.DEVEL)

        (ns, extra) = op.parse_known_args(args=line.split())

        if ns.nodefaults and ns.optional:
            raise KickstartParseError(_("Group cannot specify both --nodefaults and --optional"))

        # If the group name has spaces in it, we have to put it back together
        # now.
        grp = " ".join(extra)

        if grp in [g.name for g in self.groupList]:
            return

        if ns.nodefaults:
            self.groupList.append(Group(name=grp, include=constants.GROUP_REQUIRED))
        elif ns.optional:
            self.groupList.append(Group(name=grp, include=constants.GROUP_ALL))
        else:
            self.groupList.append(Group(name=grp, include=constants.GROUP_DEFAULT))

    def add(self, pkgList):
        """Given a list of lines from the input file, strip off any leading
           symbols and add the result to the appropriate list.
        """
        existingExcludedSet = set(self.excludedList)
        existingPackageSet = set(self.packageList)
        newExcludedSet = set()
        newPackageSet = set()

        excludedGroupList = []

        for pkg in pkgList:
            stripped = pkg.strip()

            if stripped[0:2] == "@^":
                self.environment = stripped[2:]
            elif stripped[0] == "@":
                self._processGroup(stripped[1:])
            elif stripped[0] == "-":
                if stripped[1:3] == "@^" and self.environment == stripped[3:]:
                    self.environment = None
                elif stripped[1] == "@":
                    excludedGroupList.append(Group(name=stripped[2:]))
                else:
                    newExcludedSet.add(stripped[1:])
            else:
                newPackageSet.add(stripped)

        # Groups have to be excluded in two different ways (note: can't use
        # sets here because we have to store objects):
        excludedGroupNames = [g.name for g in excludedGroupList]

        # First, an excluded group may be cancelling out a previously given
        # one.  This is often the case when using %include.  So there we should
        # just remove the group from the list.
        self.groupList = [g for g in self.groupList if g.name not in excludedGroupNames]

        # Second, the package list could have included globs which are not
        # processed by pykickstart.  In that case we need to preserve a list of
        # excluded groups so whatever tool doing package/group installation can
        # take appropriate action.
        self.excludedGroupList.extend(excludedGroupList)

        existingPackageSet = (existingPackageSet - newExcludedSet) | newPackageSet
        existingExcludedSet = (existingExcludedSet - existingPackageSet) | newExcludedSet

        # FIXME: figure these types out
        self.packageList = sorted(existingPackageSet)
        self.excludedList = sorted(existingExcludedSet)

###
### PARSER
###
class KickstartParser(object):
    """The kickstart file parser class as represented by a basic state
       machine.  To create a specialized parser, make a subclass and override
       any of the methods you care about.  Methods that don't need to do
       anything may just pass.  However, _stateMachine should never be
       overridden.
    """
    def __init__(self, handler, followIncludes=True, errorsAreFatal=True,
                 missingIncludeIsFatal=True, unknownSectionIsFatal=True):
        """Create a new KickstartParser instance.  Instance attributes:

           errorsAreFatal        -- Should errors cause processing to halt, or
                                    just print a message to the screen?  This
                                    is most useful for writing syntax checkers
                                    that may want to continue after an error is
                                    encountered.
           followIncludes        -- If %include is seen, should the included
                                    file be checked as well or skipped?
           handler               -- An instance of a BaseHandler subclass.  If
                                    None, the input file will still be parsed
                                    but no data will be saved and no commands
                                    will be executed.
           missingIncludeIsFatal -- Should missing include files be fatal, even
                                    if errorsAreFatal is False?
           unknownSectionIsFatal -- Should an unknown %section be fatal?  Not all
                                    sections are handled by pykickstart.  Some are
                                    user-defined, so there should be a way to have
                                    pykickstart ignore them.
        """
        self.errorsAreFatal = errorsAreFatal
        self.errorsCount = 0
        self.followIncludes = followIncludes
        self.handler = handler
        self.currentdir = {}
        self.missingIncludeIsFatal = missingIncludeIsFatal
        self.unknownSectionIsFatal = unknownSectionIsFatal

        self._state = STATE_COMMANDS
        self._includeDepth = 0
        self._line = ""

        self.version = self.handler.version
        Script._ver = self.version
        Packages._ver = self.version

        self._sections = {}
        self.setupSections()

    def _reset(self):
        """Reset the internal variables of the state machine for a new kickstart file."""
        self._state = STATE_COMMANDS
        self._includeDepth = 0

    def getSection(self, s):
        """Return a reference to the requested section (s must start with '%'s),
           or raise KeyError if not found.
        """
        return self._sections[s]

    def handleCommand(self, lineno, args):
        """Given the list of command and arguments, call the Version's
           dispatcher method to handle the command.  Returns the command or
           data object returned by the dispatcher.  This method may be
           overridden in a subclass if necessary.
        """
        if self.handler:
            self.handler.currentLine = self._line
            retval = self.handler.dispatcher(args, lineno)
            return retval

    def registerSection(self, obj):
        """Given an instance of a Section subclass, register the new section
           with the parser.  Calling this method means the parser will
           recognize your new section and dispatch into the given object to
           handle it.
        """
        if not obj.sectionOpen:
            raise TypeError("no sectionOpen given for section %s" % obj)

        if not obj.sectionOpen.startswith("%"):
            raise TypeError("section %s tag does not start with a %%" % obj.sectionOpen)

        self._sections[obj.sectionOpen] = obj

    def _finalize(self, obj):
        """Called at the close of a kickstart section to take any required
           actions.  Internally, this is used to add scripts once we have the
           whole body read.
        """
        obj.finalize()
        self._state = STATE_COMMANDS

    def _handleSpecialComments(self, line):
        """Kickstart recognizes a couple special comments."""
        if self._state != STATE_COMMANDS:
            return

        # Save the platform for s-c-kickstart.
        if line[:10] == "#platform=":
            self.handler.platform = self._line[10:].strip()

    def _readSection(self, lineIter, lineno):
        obj = self._sections[self._state]

        while True:
            try:
                line = next(lineIter)
                if line == "" and self._includeDepth == 0:
                    # This section ends at the end of the file.
                    if self.version >= version.F8:
                        raise KickstartParseError(_("Section %s does not end with %%end.") % obj.sectionOpen, lineno=lineno)

                    self._finalize(obj)
            except StopIteration:
                break

            lineno += 1

            # Throw away blank lines and comments, unless the section wants all
            # lines.
            if self._isBlankOrComment(line) and not obj.allLines:
                continue

            if line.lstrip().startswith("%"):
                # If we're in a script, the line may begin with "%something"
                # that's not the start of any section we recognize, but still
                # valid for that script.  So, don't do the split below unless
                # we're sure.
                possibleSectionStart = line.split()[0]
                if not self._validState(possibleSectionStart) \
                   and possibleSectionStart not in ("%end", "%include"):
                    obj.handleLine(line)
                    continue

                args = shlex.split(line)

                if args and args[0] == "%end":
                    # This is a properly terminated section.
                    self._finalize(obj)
                    break
                elif args and args[0] == "%include":
                    if len(args) == 1 or not args[1]:
                        raise KickstartParseError(lineno=lineno)

                    self._handleInclude(args[1])
                    continue
                elif args and args[0] == "%ksappend":
                    continue
                elif args and self._validState(args[0]):
                    # This is an unterminated section.
                    if self.version >= version.F8:
                        raise KickstartParseError(_("Section %s does not end with %%end.") % obj.sectionOpen, lineno=lineno)

                    # Finish up.  We do not process the header here because
                    # kicking back out to STATE_COMMANDS will ensure that happens.
                    lineIter.put(line)
                    lineno -= 1
                    self._finalize(obj)
                    break
            else:
                # This is just a line within a section.  Pass it off to whatever
                # section handles it.
                obj.handleLine(line)

        return lineno

    def _validState(self, st):
        """Is the given section tag one that has been registered with the parser?"""
        return st in list(self._sections.keys())

    def _tryFunc(self, fn):
        """Call the provided function (which doesn't take any arguments) and
           do the appropriate error handling.  If errorsAreFatal is False, this
           function will just print the exception and keep going.
        """
        try:
            fn()
        except Exception as msg:    # pylint: disable=broad-except
            self.errorsCount += 1
            if self.errorsAreFatal:
                raise
            else:
                print(msg, file=sys.stderr)

    def _isBlankOrComment(self, line):
        return line.isspace() or line == "" or line.lstrip()[0] == '#'

    def _handleInclude(self, f):
        # This case comes up primarily in ksvalidator.
        if not self.followIncludes:
            return

        self._includeDepth += 1

        try:
            self.readKickstart(f, reset=False)
        except KickstartError:
            # Handle the include file being provided over the
            # network in a %pre script.  This case comes up in the
            # early parsing in anaconda.
            if self.missingIncludeIsFatal:
                raise

        self._includeDepth -= 1

    def _stateMachine(self, lineIter):
        # For error reporting.
        lineno = 0

        while True:
            # Get the next line out of the file, quitting if this is the last line.
            try:
                self._line = next(lineIter)
                if self._line == "":
                    break
            except StopIteration:
                break

            lineno += 1

            # Eliminate blank lines, whitespace-only lines, and comments.
            if self._isBlankOrComment(self._line):
                self._handleSpecialComments(self._line)
                continue

            # Split the line, discarding comments.
            args = shlex.split(self._line, comments=True)

            if args[0] == "%include":
                if len(args) == 1 or not args[1]:
                    raise KickstartParseError(lineno=lineno)

                self._handleInclude(args[1])
                continue

            # Now on to the main event.
            if self._state == STATE_COMMANDS:
                if args[0] == "%ksappend":
                    # This is handled by the preprocess* functions, so continue.
                    continue
                elif args[0][0] == '%':
                    # This is the beginning of a new section.  Handle its header
                    # here.
                    newSection = args[0]
                    if not self._validState(newSection):
                        if self.unknownSectionIsFatal:
                            raise KickstartParseError(_("Unknown kickstart section: %s") % newSection, lineno=lineno)
                        else:
                            # If we are ignoring unknown section errors, just create a new
                            # NullSection for the header we just saw.  Then nothing else
                            # needs to change.  You can turn this warning into an error via
                            # ksvalidator, or the warnings module.
                            warnings.warn(_("Potentially unknown section seen at line %(lineno)s: %(sectionName)s") % {"lineno": lineno, "sectionName": newSection}, KickstartParseWarning)
                            self.registerSection(NullSection(self.handler, sectionOpen=newSection))

                    self._state = newSection
                    obj = self._sections[self._state]
                    self._tryFunc(lambda: obj.handleHeader(lineno, args))

                    # This will handle all section processing, kicking us back
                    # out to STATE_COMMANDS at the end with the current line
                    # being the next section header, etc.
                    lineno = self._readSection(lineIter, lineno)
                else:
                    # This is a command in the command section.  Dispatch to it.
                    self._tryFunc(lambda: self.handleCommand(lineno, args))
            elif self._state == STATE_END:
                break
            elif self._includeDepth > 0:
                lineIter.put(self._line)
                lineno -= 1
                lineno = self._readSection(lineIter, lineno)

    def readKickstartFromString(self, s, reset=True):
        """Process a kickstart file, provided as the string str."""
        if reset:
            self._reset()

        # Add a "" to the end of the list so the string reader acts like the
        # file reader and we only get StopIteration when we're after the final
        # line of input.
        i = PutBackIterator(s.splitlines(True) + [""])
        self._stateMachine(i)

    def readKickstart(self, f, reset=True):
        """Process a kickstart file, given by the filename f."""
        if reset:
            self._reset()

        # an %include might not specify a full path.  if we don't try to figure
        # out what the path should have been, then we're unable to find it
        # requiring full path specification, though, sucks.  so let's make
        # the reading "smart" by keeping track of what the path is at each
        # include depth.
        if not os.path.exists(f):
            if self._includeDepth - 1 in self.currentdir:
                if os.path.exists(os.path.join(self.currentdir[self._includeDepth - 1], f)):
                    f = os.path.join(self.currentdir[self._includeDepth - 1], f)

        cd = os.path.dirname(f)
        if not cd.startswith("/"):
            cd = os.path.abspath(cd)
        self.currentdir[self._includeDepth] = cd

        try:
            s = load_to_str(f)
        except KickstartError as e:
            raise KickstartError(_("Unable to open input kickstart file: %s") % str(e), lineno=0)

        self.readKickstartFromString(s, reset=False)

    def setupSections(self):
        """Install the sections all kickstart files support.  You may override
           this method in a subclass, but should avoid doing so unless you know
           what you're doing.
        """
        self._sections = {}

        # Install the sections all kickstart files support.
        self.registerSection(PreScriptSection(self.handler, dataObj=Script))
        self.registerSection(PreInstallScriptSection(self.handler, dataObj=Script))
        self.registerSection(PostScriptSection(self.handler, dataObj=Script))
        self.registerSection(OnErrorScriptSection(self.handler, dataObj=Script))
        self.registerSection(TracebackScriptSection(self.handler, dataObj=Script))
        self.registerSection(PackageSection(self.handler))

        # Whitelist well-known sections that pykickstart does not understand,
        # but shouldn't error on.
        self.registerSection(NullSection(self.handler, sectionOpen="%addon"))
        self.registerSection(NullSection(self.handler, sectionOpen="%anaconda"))



#
# Alexander Todorov <atodorov@redhat.com>
#
# Copyright 2008 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import F10
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class F10_Rescue(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.rescue = False
        self.nomount = kwargs.get("nomount", False)
        self.romount = kwargs.get("romount", False)

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.rescue:
            retval += "rescue"

            if self.nomount:
                retval += " --nomount"
            if self.romount:
                retval += " --romount"

            retval = "# Start rescue mode\n%s\n" % retval

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="rescue", description="""
                            Automatically enter the installer's rescue mode.
                            This gives you a chance to repair the system should
                            something catastrophic happen.""", epilog="""
                            By default, the installer will find your system and
                            mount it in read-write mode, telling you where it has
                            performed this mount. You may optionally choose to
                            not mount anything or mount in read-only mode. Only
                            one of these two options may be given at any one
                            time.""", version=F10)
        op.add_argument("--nomount", action="store_true", version=F10,
                        default=False, help="""
                        Don't mount the installed system.""")
        op.add_argument("--romount", action="store_true", version=F10,
                        default=False, help="""
                        Mount the installed system in read-only mode.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)

        if ns.nomount and ns.romount:
            raise KickstartParseError(_("Only one of --nomount and --romount may be specified for rescue command."), lineno=self.lineno)

        self.set_to_self(ns)
        self.rescue = True
        return self



import unittest

from pykickstart.errors import KickstartDeprecationWarning
from tests.baseclass import CommandTest


class FC3_TestCase(CommandTest):
    command = "authconfig"

    def runTest(self):
        # pass
        self.assert_parse("authconfig")
        self.assert_parse("authconfig --cheese", "auth --cheese\n")
        self.assert_parse("authconfig --cracker=CRUNCHY", "auth --cracker=CRUNCHY\n")
        self.assert_parse("auth")
        self.assert_parse("auth --cheese", "auth --cheese\n")
        self.assert_parse("auth --cracker=CRUNCHY", "auth --cracker=CRUNCHY\n")


class F28_TestCase(FC3_TestCase):

    def runTest(self):
        super().runTest()

        with self.assertWarns(KickstartDeprecationWarning):
            self.assert_parse("authconfig")

        with self.assertWarns(KickstartDeprecationWarning):
            self.assert_parse("auth")


if __name__ == "__main__":
    unittest.main()



#
# Copyright 2019 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#


import unittest
from tests.baseclass import CommandTest


class RHEL8_TestCase(CommandTest):
    def runTest(self):
        # basic parsing
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd"')
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd" --connect-to-insights')
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd" --server-hostname="https://rhsm.example.com"')
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd" --rhsm-baseurl="https://content.example.com"')
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd" --server-hostname="https://rhsm.example.com" --connect-to-insights')
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd" --proxy="http://proxy.com"')
        # just the rhsm command without any options is not valid
        self.assert_parse_error('rhsm')

        # multiple activation keys can be passed
        self.assert_parse('rhsm --organization="12345" --activation-key="a" --activation-key="b" --activation-key="c"')

        # at least one activation key needs to be present
        self.assert_parse_error('rhsm --organization="12345"')

        # empty string is not a valid activation key
        self.assert_parse_error('rhsm --organization="12345" --activation-key=""')
        self.assert_parse_error('rhsm --organization="12345" --activation-key="a" --activation-key="b" --activation-key=""')

        # organization id needs to be always specified
        self.assert_parse_error('rhsm --activation-key="a"')
        self.assert_parse_error('rhsm --activation-key="a" --activation-key="b" --activation-key="c"')

        # check proxy parsing
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd" --proxy="http://proxy.com"')
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd" --proxy="http://proxy.com:9001"')
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd" --proxy="http://username@proxy.com:9001"')
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd" --proxy="http://username:password@proxy.com:9001"')

        # unknown options are an error
        self.assert_parse_error('rhsm --organization="12345" --activation-key="abcd" --unknown=stuff')

        # test output kickstart generation
        # TODO: check if it is OK to have the organization name & activation key in output kickstart
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd"',
                          'rhsm --organization="12345" --activation-key="abcd"\n')
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd" --activation-key="efgh"',
                          'rhsm --organization="12345" --activation-key="abcd" --activation-key="efgh"\n')
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd" --server-hostname="https://rhsm.example.com" --connect-to-insights',
                          'rhsm --organization="12345" --activation-key="abcd" --connect-to-insights --server-hostname="https://rhsm.example.com"\n')
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd" --rhsm-baseurl="https://content.example.com" --connect-to-insights',
                          'rhsm --organization="12345" --activation-key="abcd" --connect-to-insights --rhsm-baseurl="https://content.example.com"\n')
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd" --rhsm-baseurl="https://content.example.com" --server-hostname="https://rhsm.example.com"',
                          'rhsm --organization="12345" --activation-key="abcd" --server-hostname="https://rhsm.example.com" --rhsm-baseurl="https://content.example.com"\n')
        self.assert_parse('rhsm --organization="12345" --activation-key="abcd" --proxy="http://username:password@proxy.com:9001"',
                          'rhsm --organization="12345" --activation-key="abcd" --proxy="http://username:password@proxy.com:9001"\n')

if __name__ == "__main__":
    unittest.main()



# Chris Lumens <clumens@redhat.com>
#
# Copyright 2016 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.commands.multipath import FC6_MultiPath
from pykickstart.base import DeprecatedCommand, RemovedCommand
from pykickstart.version import FC6

class FC6_TestCase(CommandTest):
    command = "multipath"

    def runTest(self):
        for action in FC6_MultiPath()._getParser()._actions:
            if '--name' in action.option_strings:
                self.assertTrue(action.required)

            for a in ['--device', '--rule']:
                if a in action.option_strings:
                    self.assertTrue(action.required)
                    self.assertTrue(action.notest)

        # pass
        self.assert_parse("multipath --name=mpath0 --device=/dev/sdc --rule=failover",
                          "multipath --name=mpath0 --device=/dev/sdc --rule=\"failover\"\n")

        # extra test coverage
        multipath = self.handler().commands["multipath"]
        multipath.dataList().append(multipath.parse("--name=mpath0 --device=/dev/sdc --rule=failover".split(" ")))
        self.assertEqual(multipath.__str__(), "multipath --name=mpath0 --device=/dev/sdc --rule=\"failover\"\n")

        # fail
        self.assert_parse_error("multipath")
        self.assert_parse_error("multipath --name")
        self.assert_parse_error("multipath --device")
        self.assert_parse_error("multipath --rule")
        self.assert_parse_error("multipath --name=mpath0")
        self.assert_parse_error("multipath --name mpath0")
        self.assert_parse_error("multipath --device /dev/sdc")
        self.assert_parse_error("multipath --device=/dev/sdc")
        self.assert_parse_error("multipath --magic")

class FC6_Duplicate_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = FC6

    def runTest(self):
        self.assert_parse_error("""
multipath --name=mpath0 --device=/dev/sda --rule=failover
multipath --name=mpath1 --device=/dev/sda --rule=failover""")

        self.assert_parse("""
multipath --name=mpath0 --device=/dev/sda --rule=failover
multipath --name=mpath0 --device=/dev/sdb --rule=failover""")

class F24_TestCase(FC6_TestCase):
    def runTest(self):
        # make sure we've been deprecated
        parser = self.getParser("multipath")
        self.assertEqual(issubclass(parser.__class__, DeprecatedCommand), True)

class F34_TestCase(F24_TestCase):
    def runTest(self):
        # make sure that dmraid is removed
        cmd = self.handler().commands["multipath"]
        self.assertTrue(issubclass(cmd.__class__, RemovedCommand))

if __name__ == "__main__":
    unittest.main()



import unittest

from pykickstart.version import makeVersion, versionMap

class Attribute_TestCase(unittest.TestCase):
    def _valid_attr(self, attr):
        # Throw out attributes that:
        # * Start with an underscore - they're probably internal use only things
        # * Are deprecated - we don't set those attributes on the object
        # * Are marked as notest - We are doing something weird with those
        # * Are positional arguments (e.g. no option_strings present)
        return not attr.dest.startswith("_") and not attr.deprecated and \
               not attr.notest and len(attr.option_strings) > 0

    def _test_one_command(self, handler, cmdClass):
        # Create an instance of the command class here so we have something
        # to inspect for attributes.  We'll throw this away after we're
        # done using it.
        cmdObj = cmdClass()
        cmdObj.handler = handler

        # Some commands are very simple and don't need a parser.  We don't
        # need to test those commands.
        if not hasattr(cmdObj, "op"):
            return

        attrs = list(set((x.dest, x.type) for x in cmdObj.op._actions if self._valid_attr(x)))
        call_kwargs = {}
        init_kwargs = {}

        # Set some known sentinel value, being careful to handle types.
        for a in attrs:
            if a[1] == int:
                call_kwargs[a[0]] = 47
                init_kwargs[a[0]] = 147
            elif a[1] == bool:
                call_kwargs[a[0]] = False
                init_kwargs[a[0]] = True
            elif getattr(a[1], "func_name", "") == "commaSplit":
                call_kwargs[a[0]] = ["call-placeholder"]
                init_kwargs[a[0]] = ["init-placeholder"]
            else:
                call_kwargs[a[0]] = "call-placeholder"
                init_kwargs[a[0]] = "init-placeholder"

        # If this is a command that can be listed more than once, instantiate its
        # data class and use that for the rest of the testing.  Otherwise, create
        # a new instance of the command class.
        if cmdObj.dataClass is not None:
            obj = cmdObj.dataClass(**init_kwargs)
        else:
            obj = cmdClass(**init_kwargs)

        del(cmdObj)

        obj.seen = True

        # Test that passing all the attributes into the __init__ method worked.
        for a in attrs:
            val = getattr(obj, a[0])
            if a[1] == int:
                self.assertEqual(val, 147, "init: %s got %s, expected 147" % (a[0], val))
            elif a[1] == bool:
                self.assertTrue(val, "init: %s got %s, expected True" % (a[0], val))
            else:
                self.assertEqual(val, "init-placeholder", "init: %s got %s, expected 'init-placeholder'" % (a[0], val))

        # And then test that passing in all the attributes via the __call__
        # method also worked.
        obj(**call_kwargs)

        for a in attrs:
            val = getattr(obj, a[0])
            if a[1] == int:
                self.assertEqual(val, 47, "init: %s got %s, expected 47" % (a[0], val))
            elif a[1] == bool:
                self.assertFalse(val, "init: %s got %s, expected False" % (a[0], val))
            else:
                self.assertEqual(val, "call-placeholder", "init: %s got %s, expected 'call-placeholder'" % (a[0], val))

    def runTest(self):
        for ver in versionMap.values():
            handler = makeVersion(ver)
            for cmdClass in (v for (k, v) in handler.commandMap.items() if k != "method"):
                self._test_one_command(handler, cmdClass)



import os
import sys
import unittest
import importlib
import unittest.mock as mock
from argparse import Namespace

from tests.baseclass import ParserTest
import pykickstart
from pykickstart.parser import Script, KickstartParser
from pykickstart.version import F25
from pykickstart.handlers.f25 import F25Handler
from pykickstart.constants import KS_SCRIPT_POST
from pykickstart.errors import KickstartParseError
from pykickstart.commands.zfcp import F14_ZFCPData
from pykickstart.commands.autopart import F23_AutoPart
from pykickstart.commands.btrfs import F17_BTRFS, F23_BTRFS, F23_BTRFSData
from pykickstart.commands.cdrom import FC3_Cdrom
from pykickstart.base import BaseData, BaseHandler, DeprecatedCommand, KickstartCommand
from pykickstart.base import KickstartHandler, RemovedCommand


class KickstartCommandWithRemovals(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords + ["connect"]
    removedAttrs = KickstartCommand.removedAttrs + ["connect"]

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)

class KickstartCommandRemovedKeywords_TestCase(unittest.TestCase):
    @mock.patch('warnings.warn')
    def runTest(self, _warn):
        KickstartCommandWithRemovals(connect='test')
        self.assertEqual(_warn.call_count, 1)

        cmd = KickstartCommandWithRemovals()
        cmd.__call__(connect='test', missingAttr='test')
        self.assertFalse(hasattr(cmd, 'connect'))
        self.assertFalse(hasattr(cmd, 'missingAttr'))

class BaseDataWithRemovals(BaseData):
    removedKeywords = BaseData.removedKeywords + ["connect"]
    removedAttrs = BaseData.removedAttrs + ["connect"]

    def __init__(self, writePriority=0, *args, **kwargs):
        BaseData.__init__(self, writePriority, *args, **kwargs)

class BaseDataRemovedKeywords_TestCase(unittest.TestCase):
    @mock.patch('warnings.warn')
    def runTest(self, _warn):
        BaseDataWithRemovals(connect='test')
        self.assertEqual(_warn.call_count, 1)

        data = BaseDataWithRemovals()
        data.__call__(connect='test-me')
        self.assertFalse(hasattr(data, 'connect'))

class DeleteRemovedAttrs_TestCase(unittest.TestCase):
    """
        Test if self.deleteRemovedAttrs() has been called
        and that no removed attributes remain in the object.
    """
    def runTest(self):
        errors = []
        commands_dir = os.path.join(os.path.dirname(pykickstart.__file__), "commands")
        commands_dir = os.path.abspath(commands_dir)

        self.assertTrue(os.path.exists(commands_dir))
        if commands_dir not in sys.path:
            sys.path.append(commands_dir)

        for _dirpath, _dirnames, paths in os.walk(commands_dir):
            for path in paths:
                if path == '__init__.py' or not path.endswith('.py'):
                    continue

                # load the module defining all possible command implementations
                command_module = importlib.import_module(path.replace(".py", ""))

                for _impl_name, impl_class in command_module.__dict__.items():
                    # skip everything which isn't a class
                    if type(impl_class) is not type:
                        continue

                    # skip everything which doesn't inherit
                    # from KickstartCommand or BaseData
                    if not (issubclass(impl_class, KickstartCommand) or issubclass(impl_class, BaseData)):
                        continue

                    # skip base classes as well
                    if impl_class.__name__ in ['KickstartCommand', 'DeprecatedCommand', 'RemovedCommand']:
                        continue

                    if impl_class.removedAttrs:
                        cmd = impl_class()
                        for attr in cmd.removedAttrs:
                            if hasattr(cmd, attr):
                                errors.append("%s.%s not removed, execute self.deleteRemovedAttrs()!" %
                                              (impl_class.__name__, attr))

        self.assertEqual(errors, [])


class KickstartCommandNoParseMethod(KickstartCommand):
    """ The parse() method is not defined """

class TestDeprecatedCommand(DeprecatedCommand):
    pass

class TestRemovedCommand(RemovedCommand):
    pass

class TestBaseData(BaseData):
    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.testAttr = kwargs.get('testAttr', '')

class BaseClasses_TestCase(ParserTest):
    def runTest(self):
        # fail - can't instantiate these directly
        self.assertRaises(TypeError, KickstartCommand, (100, ))
        self.assertRaises(TypeError, DeprecatedCommand, (100, ))
        self.assertRaises(TypeError, RemovedCommand, (100, ))
        self.assertRaises(TypeError, BaseHandler, (100, ))
        self.assertRaises(TypeError, BaseData, (100, ))

        cmd = KickstartCommandNoParseMethod()
        self.assertEqual(cmd.dataList(), None)
        self.assertEqual(cmd.dataClass, None)
        with self.assertRaises(TypeError):
            cmd.parse(['test'])

        with mock.patch('warnings.warn') as _warn:
            cmd._setToSelf(Namespace())
            self.assertEqual(_warn.call_count, 1)

        with mock.patch('warnings.warn') as _warn:
            cmd._setToObj(Namespace(), cmd)
            self.assertEqual(_warn.call_count, 1)

        dep_cmd = TestDeprecatedCommand()
        self.assertEqual(dep_cmd.__str__(), '')
        with mock.patch('warnings.warn') as _warn:
            dep_cmd.parse(['test'])
            self.assertEqual(_warn.call_count, 1)

        removed_cmd = TestRemovedCommand()
        self.assertEqual(removed_cmd.__str__(), '')
        with self.assertRaises(KickstartParseError):
            removed_cmd.parse(['test'])


        self.assertEqual(TestBaseData().__str__(), '')

        data = TestBaseData()
        data.__call__(testAttr='test-me', missingAttr='missing')
        self.assertEqual(data.testAttr, 'test-me')
        self.assertFalse(hasattr(data, 'missingAttr'))

class KickstartHandler_TestCase(unittest.TestCase):
    def runTest(self):
        handler = KickstartHandler()
        self.assertEqual(str(handler), "")

        handler = KickstartHandler()
        handler.registerCommand('autopart', F23_AutoPart)
        handler.registerCommand('btrfs', F17_BTRFS)
        handler.registerData('BTRFSData', F23_BTRFSData)
        handler.registerData('ZFCPData', F14_ZFCPData)

        self.assertEqual(len(handler.commands.keys()), 2)
        self.assertTrue(isinstance(handler.commands['autopart'], F23_AutoPart))
        self.assertTrue(isinstance(handler.commands['btrfs'], F17_BTRFS))

        self.assertTrue(hasattr(handler, 'BTRFSData'))
        self.assertEqual(getattr(handler, 'BTRFSData'), F23_BTRFSData)
        self.assertTrue(hasattr(handler, 'ZFCPData'))
        self.assertEqual(getattr(handler, 'ZFCPData'), F14_ZFCPData)

        handler = KickstartHandler()
        handler.registerCommand('cdrom', FC3_Cdrom)
        handler.version = F25

        parser = KickstartParser(handler)
        parser.readKickstartFromString("cdrom")
        self.assertEqual(str(handler), "# Use CDROM installation media\ncdrom\n")

class HandlerRegisterCommands_TestCase(unittest.TestCase):
    def runTest(self):
        handler = F25Handler(mapping={
                                'autopart': F23_AutoPart,
                                'btrfs': F23_BTRFS,
                             },
                             dataMapping={
                                'BTRFSData': F23_BTRFSData,
                                'ZFCPData': None,
                             },
                             commandUpdates={
                                'btrfs': F17_BTRFS,
                             },
                             dataUpdates={
                                'ZFCPData': F14_ZFCPData,
                             })
        self.assertEqual(len(handler.commands.keys()), 2)
        self.assertTrue(isinstance(handler.commands['autopart'], F23_AutoPart))
        self.assertTrue(isinstance(handler.commands['btrfs'], F17_BTRFS))

        self.assertTrue(hasattr(handler, 'BTRFSData'))
        self.assertEqual(getattr(handler, 'BTRFSData'), F23_BTRFSData)
        self.assertTrue(hasattr(handler, 'ZFCPData'))
        self.assertEqual(getattr(handler, 'ZFCPData'), F14_ZFCPData)


class HandlerString_TestCase(ParserTest):
    def runTest(self):
        self.handler.platform = "x86_64"
        self.assertIn("#platform=x86_64", str(self.handler))

        self.handler.platform = ""
        self.assertNotIn("#platform", str(self.handler))

        self.handler.scripts.append(Script("echo Hello", type=KS_SCRIPT_POST))
        self.assertIn("echo Hello", str(self.handler))

class HandlerResetCommand_TestCase(ParserTest):
    def runTest(self):
        # fail - tried to reset a command that doesn't exist
        self.assertRaises(KeyError, self.handler.resetCommand, "fakecommand")

        # might as well test this way of getting the same information, while we're at it
        self.assertFalse(self.handler.hasCommand("fakecommand"))

        # Set some attributes on a command (both by calling the command and by setting them
        # directly), verify they're set, then reset it and verify they're gone.
        self.handler.autopart(autopart=True, encrypted=True, passphrase="something")
        self.handler.autopart.cipher = "whatever"
        self.handler.autopart.bogus = True

        self.assertTrue(self.handler.autopart.autopart)
        self.assertEqual(self.handler.autopart.cipher, "whatever")
        self.assertTrue(self.handler.autopart.encrypted)
        self.assertEqual(self.handler.autopart.passphrase, "something")
        self.assertTrue(self.handler.autopart.bogus)
        self.assertTrue("autopart" in str(self.handler))

        self.handler.resetCommand("autopart")
        self.assertFalse(self.handler.autopart.autopart)
        self.assertEqual(self.handler.autopart.cipher, "")
        self.assertFalse(self.handler.autopart.encrypted)
        self.assertEqual(self.handler.autopart.passphrase, "")
        self.assertNotIn("bogus", self.handler.autopart.__dict__)
        self.assertFalse("autopart" in str(self.handler))

class HandlerDispatch_TestCase(ParserTest):
    def runTest(self):
        # fail - no such command
        self.assertRaises(KickstartParseError, self.handler.dispatcher, ["fakecommand"], 1)

        # pass - parses a valid kickstart
        self.handler.dispatcher(['autopart'], 1)
        self.assertTrue(self.handler.autopart.seen)

        self.handler.dispatcher(['network', '--device', 'eth0'], 1)
        self.assertEqual(self.handler.network.dataList()[0].device, 'eth0')


class HandlerMask_TestCase(ParserTest):
    def runTest(self):
        # First, verify all commands are set to something other than None.  This means
        # the command can be called.
        for cmd in self.handler.commands:
            self.assertIsNotNone(self.handler.commands[cmd])

        # Now blank out most commands except the one or two we're actually interested
        # in.  This has the effect of making the other commands recognized, but calling
        # them will have no effect on data.
        lst = ["rootpw", "user", "group"]

        self.handler.maskAllExcept(lst)
        for cmd in self.handler.commands:
            if cmd in lst:
                self.assertIsNotNone(self.handler.commands[cmd])
            else:
                self.assertIsNone(self.handler.commands[cmd])

        # These attributes should still be set to their defaults.
        self.handler.dispatcher(["autopart", "--encrypted", "--passphrase", "something"], 1)
        self.assertFalse(self.handler.autopart.encrypted)
        self.assertEqual(self.handler.autopart.passphrase, "")

if __name__ == "__main__":
    unittest.main()



#
# Martin Gracik <mgracik@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest

class FC6_TestCase(CommandTest):
    command = "iscsi"

    def runTest(self):
        # pass
        self.assert_parse("iscsi --ipaddr=1.1.1.1", "iscsi --ipaddr=1.1.1.1\n")
        self.assert_parse("iscsi --ipaddr=1.1.1.1 --target=tar --port=1234 --user=name --password=secret",
                          "iscsi --target=tar --ipaddr=1.1.1.1 --port=1234 --user=name --password=secret\n")
        self.assert_parse("iscsi --ipaddr=1.1.1.1 --target=tar", "iscsi --target=tar --ipaddr=1.1.1.1\n")
        self.assert_parse("iscsi --ipaddr=1.1.1.1 --port=4321", "iscsi --ipaddr=1.1.1.1 --port=4321\n")
        self.assert_parse("iscsi --ipaddr=1.1.1.1 --user=name", "iscsi --ipaddr=1.1.1.1 --user=name\n")
        self.assert_parse("iscsi --ipaddr=1.1.1.1 --password=secret", "iscsi --ipaddr=1.1.1.1 --password=secret\n")

        # fail
        # missing required option --ipaddr
        self.assert_parse_error("iscsi")
        self.assert_parse_error("iscsi --target=tar --user=name --password=secret --port=1234")
        # missing --ipaddr argument
        self.assert_parse_error("iscsi --ipaddr")
        # unexpected arguments
        self.assert_parse_error("iscsi --ipaddr=1.2.3.4 not expected")
        # unknown flag
        self.assert_parse_error("iscsi --ipaddr=1.2.3.4 --unknown=value")
        # empty arguments
        self.assert_parse_error("iscsi --target --ipaddr=1.2.3.4")
        self.assert_parse_error("iscsi --ipaddr=1.2.3.4 --user")
        self.assert_parse_error("iscsi --ipaddr=1.2.3.4 --password")
        self.assert_parse_error("iscsi --ipaddr=1.2.3.4 --port")
        self.assert_parse_error("iscsi --ipaddr=1.2.3.4 --port=''")

        # extra test coverage
        data = self.handler().IscsiData()
        data.ipaddr = ""
        self.assertEqual(data._getArgsAsStr(), "")

        cmd = self.handler().commands[self.command]
        cmd.iscsi= [data]
        self.assertEqual(cmd.__str__(), "iscsi\n")
        self.assertEqual(cmd.dataList(), [data])

class F10_TestCase(FC6_TestCase):
    def runTest(self):
        # run FC6 test case
        FC6_TestCase.runTest(self)

        # pass
        self.assert_parse("iscsi --ipaddr=1.1.1.1 --reverse-user=name --reverse-password=secret",
                          "iscsi --ipaddr=1.1.1.1 --reverse-user=name --reverse-password=secret\n")
        self.assert_parse("iscsi --ipaddr=1.1.1.1 --reverse-user=name", "iscsi --ipaddr=1.1.1.1 --reverse-user=name\n")
        self.assert_parse("iscsi --ipaddr=1.1.1.1 --reverse-password=secret", "iscsi --ipaddr=1.1.1.1 --reverse-password=secret\n")

        # fail
        # empty arguments
        self.assert_parse_error("iscsi --ipaddr=1.1.1.1 --reverse-user")
        self.assert_parse_error("iscsi --ipaddr=1.1.1.1 --reverse-password")

class RHEL6_TestCase(F10_TestCase):
    def runTest(self):
        F10_TestCase.runTest(self)

        self.assert_parse("iscsi --ipaddr=1.1.1.1 --iface=eth0\n")

        # extra test coverage
        data = self.handler().IscsiData()
        data.iface = "eth0"
        self.assertEqual(data._getArgsAsStr(), " --iface=eth0")

class F17_TestCase(F10_TestCase):
    def runTest(self):
        F10_TestCase.runTest(self)

        self.assert_parse("iscsi --ipaddr=1.1.1.1 --iface=eth0\n")

        # extra test coverage
        data = self.handler().IscsiData()
        data.iface = "eth0"
        self.assertEqual(data._getArgsAsStr(), " --iface=eth0")

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, F29, F34, versionToLongString
from pykickstart.base import KickstartCommand, DeprecatedCommand, RemovedCommand
from pykickstart.options import KSOptionParser


class FC3_DeviceProbe(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.deviceprobe = kwargs.get("deviceprobe", "")

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.deviceprobe:
            retval += "deviceprobe %s\n" % self.deviceprobe

        return retval

    def parse(self, args):
        self.deviceprobe = " ".join(args)
        return self

    def _getParser(self):
        return KSOptionParser(prog="deviceprobe", version=FC3, description="probe for devices")


class F29_DeviceProbe(DeprecatedCommand, FC3_DeviceProbe):

    def __init__(self):  # pylint: disable=super-init-not-called
        DeprecatedCommand.__init__(self)

    def _getParser(self):
        op = FC3_DeviceProbe._getParser(self)
        op.description += "\n\n.. deprecated:: %s" % versionToLongString(F29)
        return op

class F34_DeviceProbe(RemovedCommand, F29_DeviceProbe):
    def _getParser(self):
        op = F29_DeviceProbe._getParser(self)
        op.description += "\n\n.. versionremoved:: %s" % versionToLongString(F34)
        return op



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#
import warnings
from textwrap import dedent

from pykickstart.errors import KickstartDeprecationWarning
from pykickstart.version import FC3, versionToLongString, F28
from pykickstart.base import KickstartCommand
from pykickstart.options import KSOptionParser


class FC3_Authconfig(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.authconfig = kwargs.get("authconfig", "")

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.authconfig:
            retval += "# System authorization information\nauth %s\n" % self.authconfig

        return retval

    def parse(self, args):
        self.authconfig = self.currentLine[len(self.currentCmd):].strip()
        return self

    def _getParser(self):
        op = KSOptionParser(prog="auth|authconfig",  description="""
                            This required command sets up the authentication
                            options for the system. This is just a wrapper
                            around the authconfig program, so all options
                            recognized by that program are valid for this
                            command. See the manual page for authconfig for a
                            complete list.

                            By default, passwords are normally encrypted and
                            are not shadowed.""", version=FC3)
        op.add_argument("options", metavar="[options]", help="""
                        See ``man authconfig``.""", version=FC3)
        return op


class F28_Authconfig(FC3_Authconfig):
    removedKeywords = FC3_Authconfig.removedKeywords
    removedAttrs = FC3_Authconfig.removedAttrs

    def parse(self, args):
        warnings.warn("The authconfig command will be deprecated, use authselect "
                      "instead.", KickstartDeprecationWarning)

        return super(F28_Authconfig, self).parse(args)

    def _getParser(self):
        op = super(F28_Authconfig, self)._getParser()
        op.description += dedent("""

            .. versionchanged:: %s

            The authconfig program is deprecated. This command will use the
            authconfig compatibility tool, but you should use the authselect
            command instead.

        """ % versionToLongString(F28))
        return op



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, F9, F10, F14, F20, F28
from pykickstart.base import KickstartCommand
from pykickstart.options import ExtendAction, ExtendConstAction, KSOptionParser, commaSplit

class FC3_Firewall(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.enabled = kwargs.get("enabled", None)
        self.ports = kwargs.get("ports", [])
        self.trusts = kwargs.get("trusts", [])

    def __str__(self):
        extra = []
        filteredPorts = []

        retval = KickstartCommand.__str__(self)

        if self.enabled is None:
            return retval

        if self.enabled:
            # It's possible we have words in the ports list instead of
            # port:proto (s-c-kickstart may do this).  So, filter those
            # out into their own list leaving what we expect.
            for port in self.ports:
                if port == "ssh:tcp":
                    extra.append(" --ssh")
                elif port == "telnet:tcp":
                    extra.append(" --telnet")
                elif port == "smtp:tcp":
                    extra.append(" --smtp")
                elif port == "http:tcp":
                    extra.append(" --http")
                elif port == "ftp:tcp":
                    extra.append(" --ftp")
                else:
                    filteredPorts.append(port)

            # All the port:proto strings go into a comma-separated list.
            portstr = ",".join(filteredPorts)
            if portstr:
                portstr = " --port=" + portstr

            extrastr = "".join(extra)
            truststr = ",".join(self.trusts)

            if truststr:
                truststr = " --trust=" + truststr

            # The output port list consists only of port:proto for
            # everything that we don't recognize, and special options for
            # those that we do.
            retval += "# Firewall configuration\nfirewall --enabled%s%s%s\n" % (extrastr, portstr, truststr)
        else:
            retval += "# Firewall configuration\nfirewall --disabled\n"

        return retval

    def _getParser(self):
        def firewall_port_cb(value):
            retval = []
            for p in value.split(","):
                p = p.strip()
                if p.find(":") == -1:
                    p = "%s:tcp" % p

                retval.append(p)

            return retval

        op = KSOptionParser(prog="firewall", description="""
                            This option corresponds to the Firewall Configuration
                            screen in the installation program""",
                            version=FC3)
        op.add_argument("--disable", "--disabled", dest="enabled",
                        action="store_false", version=FC3,
                        help="Do not configure any iptables rules.")
        op.add_argument("--enable", "--enabled", dest="enabled",
                        action="store_true", default=True, help="""
                        Reject incoming connections that are not in response
                        to outbound requests, such as DNS replies or DHCP
                        requests. If access to services running on this machine
                        is needed, you can choose to allow specific services
                        through the firewall.""", version=FC3)
        op.add_argument("--ftp", dest="ports", action="append_const",
                        const="21:tcp", version=FC3,
                        help="Open port 21:tcp")
        op.add_argument("--http", dest="ports", action=ExtendConstAction,
                        const=["80:tcp", "443:tcp"], nargs=0, version=FC3,
                        help="Open ports 80:tcp and 443:tcp")
        op.add_argument("--smtp", dest="ports", action="append_const",
                        const="25:tcp", version=FC3,
                        help="Open port 25:tcp")
        op.add_argument("--ssh", dest="ports", action="append_const",
                        const="22:tcp", version=FC3,
                        help="Open port 22:tcp")
        op.add_argument("--telnet", dest="ports", action="append_const",
                        const="23:tcp", version=FC3,
                        help="Open port 23:tcp")
        op.add_argument("--high", deprecated=FC3, help="")
        op.add_argument("--medium", deprecated=FC3, help="")
        op.add_argument("--port", dest="ports", action=ExtendAction,
                        type=firewall_port_cb, help="""
                        You can specify that ports be allowed through the firewall
                        using the port:protocol format. You can also specify ports
                        numerically. Multiple ports can be combined into one option
                        as long as they are separated by commas. For example::

                        ``firewall --port=imap:tcp,1234:ucp,47``""",
                        version=FC3)
        op.add_argument("--trust", dest="trusts", action="append", help="""
                        Listing a device here, such as eth0, allows all traffic
                        coming from that device to go through the firewall. To
                        list more than one device, use --trust eth0 --trust eth1.
                        Do NOT use a comma-separated format such as
                        --trust eth0, eth1.""", version=FC3)
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        return self

class F9_Firewall(FC3_Firewall):
    removedKeywords = FC3_Firewall.removedKeywords
    removedAttrs = FC3_Firewall.removedAttrs

    def _getParser(self):
        op = FC3_Firewall._getParser(self)
        op.remove_argument("--high", version=F9)
        op.remove_argument("--medium", version=F9)
        return op

class F10_Firewall(F9_Firewall):
    removedKeywords = F9_Firewall.removedKeywords
    removedAttrs = F9_Firewall.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        F9_Firewall.__init__(self, writePriority, *args, **kwargs)
        self.services = kwargs.get("services", [])

    def __str__(self):
        if self.enabled is None:
            return ""

        retval = F9_Firewall.__str__(self)
        if self.enabled:
            retval = retval.strip()

            svcstr = ",".join(self.services)
            if svcstr:
                svcstr = " --service=" + svcstr

            return retval + "%s\n" % svcstr
        else:
            return retval

    def _getParser(self):
        op = F9_Firewall._getParser(self)
        op.add_argument("--service", dest="services", action=ExtendAction,
                        type=commaSplit, help="""
                        This option provides a higher-level way to allow services
                        through the firewall. Some services (like cups, avahi, etc.)
                        require multiple ports to be open or other special
                        configuration in order for the service to work. You could
                        specify each individual service with the ``--port`` option,
                        or specify ``--service=`` and open them all at once.

                        Valid options are anything recognized by the
                        firewall-cmd program in the firewalld package.
                        If firewalld is running::

                        ``firewall-cmd --get-services``

                        will provide a list of known service names.""",
                        version=F10)
        op.add_argument("--ftp", dest="services", action="append_const",
                        const="ftp", version=F10, help="Open the ftp service port")
        op.add_argument("--http", dest="services", action="append_const",
                        const="http", version=F10, help="Open the http service port")
        op.add_argument("--smtp", dest="services", action="append_const",
                        const="smtp", version=F10, help="Open the smtp service port")
        op.add_argument("--ssh", dest="services", action="append_const",
                        const="ssh", version=F10, help="Open the ssh service port")
        op.add_argument("--telnet", deprecated=F10)
        return op

class F14_Firewall(F10_Firewall):
    removedKeywords = F10_Firewall.removedKeywords + ["telnet"]
    removedAttrs = F10_Firewall.removedAttrs + ["telnet"]

    def _getParser(self):
        op = F10_Firewall._getParser(self)
        op.remove_argument("--telnet", version=F14)
        return op

class F20_Firewall(F14_Firewall):
    def __init__(self, writePriority=0, *args, **kwargs):
        F14_Firewall.__init__(self, writePriority, *args, **kwargs)
        self.remove_services = kwargs.get("remove_services", [])

    def _getParser(self):
        op = F14_Firewall._getParser(self)
        op.add_argument("--remove-service", dest="remove_services",
                        help="Close ports for the comma-separated list of services",
                        action=ExtendAction, type=commaSplit, version=F20)
        return op

    def __str__(self):
        if self.enabled is None:
            return ""

        retval = F14_Firewall.__str__(self)
        if self.enabled:
            retval = retval.strip()

            svcstr = ",".join(self.remove_services)
            if svcstr:
                svcstr = " --remove-service=" + svcstr

            return retval + "%s\n" % svcstr
        else:
            return retval

class F28_Firewall(F20_Firewall):
    def __init__(self, writePriority=0, *args, **kwargs):
        F20_Firewall.__init__(self, writePriority, *args, **kwargs)
        self.use_system_defaults = kwargs.get("use_system_defaults", None)

    def _getParser(self):
        op = F20_Firewall._getParser(self)
        op.add_argument("--use-system-defaults", dest="use_system_defaults",
                        action="store_true", default=False, version=F28, help="""
                        Don't configure the firewall at all. This instructs anaconda
                        to do nothing and allows the system to rely on the defaults
                        that were provided with the package or ostree.  If this option
                        is used with other options then all other options will be
                        ignored.""")
        return op

    def __str__(self):
        if self.use_system_defaults:
            return "# Firewall configuration\nfirewall --use-system-defaults\n"
        else:
            return F20_Firewall.__str__(self)



import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.clearpart import FC3_ClearPart

class FC3_TestCase(CommandTest):
    command = "clearpart"

    def runTest(self):
        cmd = FC3_ClearPart()
        self.assertFalse(cmd.initAll)

        cmd.type = -1 # invalid value, don't know what partitions to clear
        self.assertEqual(cmd.__str__(), "# Partition clearing information\nclearpart\n")

        # pass
        self.assert_parse("clearpart")
        self.assert_parse("clearpart --all", "clearpart --all\n")
        self.assert_parse("clearpart --none", "clearpart --none\n")
        # Passing multiple competing type options should accept only the last one
        self.assert_parse("clearpart --linux --none --all", "clearpart --all\n")
        # Setting --initlabel or --drives without a type option should 'fail'
        self.assert_parse("clearpart --initlabel", "")
        self.assert_parse("clearpart --drives sda", "")

        self.assert_parse("clearpart --all --initlabel", "clearpart --all --initlabel\n")
        self.assert_parse("clearpart --all --drives sda", "clearpart --all --drives=sda\n")
        self.assert_parse("clearpart --all --drives sda,sdb", "clearpart --all --drives=sda,sdb\n")
        self.assert_parse("clearpart --all --drives=sda", "clearpart --all --drives=sda\n")
        self.assert_parse("clearpart --all --drives=sda,sdb", "clearpart --all --drives=sda,sdb\n")
        # Big Everything Test
        self.assert_parse("clearpart --drives=sda,sdb --all --linux --initlabel", "clearpart --linux --initlabel --drives=sda,sdb\n")

        # fail
        # initlabel should not take a value
        self.assert_parse_error("clearpart --initlabel=foo")
        # drives must take a value
        self.assert_parse_error("clearpart --all --drives")
        # nonsensical parameter test
        self.assert_parse_error("clearpart --cheese")

class F17_TestCase(FC3_TestCase):
    def runTest(self):
        FC3_TestCase.runTest(self)
        self.assert_parse("clearpart --list=sda2,sda3,disk/by-label/foo",
                          "clearpart --list=sda2,sda3,disk/by-label/foo\n")

class F21_TestCase(F17_TestCase):
    def runTest(self):
        F17_TestCase.runTest(self)
        self.assert_parse("clearpart --all --initlabel --disklabel=gpt",
                          "clearpart --all --initlabel --disklabel=gpt\n")
        self.assert_parse_error("clearpart --all --disklabel")

class F28_TestCase(F21_TestCase):
    def runTest(self):
        F21_TestCase.runTest(self)
        self.assert_parse("clearpart --all --cdl",
                          "clearpart --all --cdl\n")

        self.assert_parse("clearpart --all --drives=dasda,dasdb,dasdc --cdl",
                          "clearpart --all --drives=dasda,dasdb,dasdc --cdl\n")

        # cdl should not take a value
        self.assert_parse_error("clearpart --cdl=foo")

class RHEL7_TestCase(F28_TestCase):
    def runTest(self):
        F28_TestCase.runTest(self)


if __name__ == "__main__":
    unittest.main()



import warnings

from tests.baseclass import ParserTest

from pykickstart.errors import KickstartDeprecationWarning
from pykickstart.parser import KickstartParser
from pykickstart.version import makeVersion, F33


class TracebackScriptSection_TestCase(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = "\n%traceback\n%end\n"

    def runTest(self):

        # F33 has no warnings
        handler = makeVersion(version=F33)
        parser = KickstartParser(handler)
        with warnings.catch_warnings(record=True) as w:
            parser.readKickstartFromString(self.ks)
            self.assertEqual(len(w), 0)

        # F34 warns about deprecation
        with self.assertWarns(KickstartDeprecationWarning):
            self.parser.readKickstartFromString(self.ks)
